@@ -63,7 +63,7 @@
 #define is_upper(x)	((x >= 'A') && (x <= 'Z'))
 #define is_cname_char(x) ((x == '-') || ((x >= '0') && (x <= '9')) \
 			|| ((x >= 'A') && (x <= 'Z')) || (x == '_') \
-			|| ((x >= 'a') && (x <= 'z')))
+			|| ((x >= 'a') && (x <= 'z')) || (x == '.'))
 #define is_ascii(x)	((x >= 0x21) && (x < 0x7f))
 
 #define SKFtolower(x)	(((x>='A')&&(x<='Z'))? (x+0x20) : x)
@@ -84,10 +84,13 @@ extern int	be_in P_((skfFILE *));
 extern int	esc_process P_((skfFILE *));
 extern int	c1_process P_((skfFILE *, int));
 extern int	seq_sweep P_((skfFILE *, int));
-extern int	is_idnspace P_((skf_ucode));
 
 extern int	cns11643_defined_code;
 
+#ifdef FUTURESUPPORT
+extern int	is_idnspace P_((skf_ucode));
+#endif
+
 /* --------------------------------------------------------------- */
 extern int preConvert P_((skfFILE *));
 
@@ -98,6 +101,7 @@ extern void g0table2low();	/* psuedo macro			   */
 extern void g1table2low();
 extern void g2table2low();
 extern void g3table2low();
+extern void g0table2up();
 extern void g1table2up();
 extern void g2table2up();
 extern void g3table2up();
@@ -130,6 +134,7 @@ extern void clear_after_mime();
 /* table len must not set below 128 (no check in e_in for speed)   */
 /*  charwidth:	1	unibyte table and table entry is short	   */
 /*  		2	multibyte table and table entry is short   */
+/*  		3	unibyte table and table entry is long      */
 /*  		4	multibyte table and table entry is long	   */
 /* --------------------------------------------------------------- */
 struct iso_byte_defs {		/* coded charset definitions	   */
@@ -152,9 +157,13 @@ extern struct iso_byte_defs iso_iso8859_defs[];
 extern struct iso_byte_defs iso_3_dblbyte_defs[];
 extern struct iso_byte_defs iso_4_dblbyte_defs[];
 extern struct iso_byte_defs misc_byte_defs[];
+extern struct iso_byte_defs miscmul_byte_defs[];
 extern struct iso_byte_defs cp_byte_defs[];
 extern struct iso_byte_defs ovlay_byte_defs[];
 extern struct iso_byte_defs priv_dblbyte_defs[];
+extern struct iso_byte_defs priv_byte_defs[];
+extern struct iso_byte_defs priv_drcs_byte_defs[];
+extern struct iso_byte_defs priv_drcs_dblbyte_defs[];
 
 /* --------------------------------------------------------------- */
 /* is_kana flags 						   */
@@ -225,6 +234,12 @@ extern struct iso_byte_defs priv_dblbyte_defs[];
 #define JOHAB_HANJA1	0xe0
 #define JOHAB_HANJA1E	0xf9
 
+#define is_multibyte(x)	(((x & 0x01) == 0) && (x != 0))
+#define is_singlebyte(x) ((x & 0x01) == 1)
+#define is_tbllong(x)	(x > 2)
+#define is_tblshort(x)	(x < 3)
+#define is_tblundef(x)	((x == 0) || (x >= 5))
+
 /* --------------------------------------------------------------- */
 /* input codetable predefinitions				   */
 /* --------------------------------------------------------------- */
@@ -361,18 +376,20 @@ extern int has_mime;		/* true if detect input mime	   */
 #define BV_SED  0xe2
 #define BV_ESC  0xff
 
-extern struct iso_byte_defs bright_byte_defs[];
-
 /* --------------------------------------------------------------- */
+#if 0
 extern unsigned short uni_t_k21[];
 extern unsigned short uni_t_k22[];
+#endif
 
 extern unsigned short uni_t_x208[];
+#if 0
 extern unsigned short uni_t_dia[];
 extern unsigned short uni_t_xsign[];
 extern unsigned short uni_t_xgreek[];
 extern unsigned short uni_t_xrcap[];
 extern unsigned short uni_t_xrom[];
+#endif
 extern unsigned short uni_t_x212[];
 
 extern unsigned short *uni_t_sjisx208;
@@ -396,6 +413,8 @@ extern unsigned long sshift_condition;	/* single shift specific   */
 #define	IN_SS1_FLG	0x00000100UL
 #define	IN_SS2_FLG	0x00000200UL
 #define	IN_SS3_FLG	0x00000400UL
+#define	IN_MACROL_FLG	0x00010000UL	/* macro is in GL	   */
+#define	IN_MACROR_FLG	0x00020000UL	/* macro is in GR	   */
 #define	IN_221_2_FLG	0x01000000UL	/* x-0221: 2 octet	   */
 #define	IN_221_4_FLG	0x02000000UL	/* x-0221: 4 octet	   */
 
@@ -413,7 +432,7 @@ extern unsigned long sshift_condition;	/* single shift specific   */
 #define in_locking_shift	(shift_condition & 0x000000ffUL)
 #define in_left_locking_shift	(shift_condition & 0x0000000fUL)
 #define in_right_locking_shift	(shift_condition & 0x000000f0UL)
-#define in_single_shift		(sshift_condition != 0)
+#define in_single_shift		(sshift_condition & 0x00030f00UL)
 
 #define set_ls1	   shift_condition = \
 	(shift_condition & 0xffffff0fUL) | IN_LS1_FLG
@@ -443,31 +462,48 @@ extern unsigned long sshift_condition;	/* single shift specific   */
 #define res_ss2	   sshift_condition &= (0xffffffffUL ^ IN_SS2_FLG)
 #define res_ss3	   sshift_condition &= (0xffffffffUL ^ IN_SS3_FLG)
 
+#define is_macl    (sshift_condition & IN_MACROL_FLG)
+#define is_macr    (sshift_condition & IN_MACROR_FLG)
+#define set_macl   sshift_condition |= IN_MACROL_FLG
+#define set_macr   sshift_condition |= IN_MACROR_FLG
+#define res_macl   sshift_condition &= (0xffffffffUL ^ IN_MACROL_FLG)
+#define res_macr   sshift_condition &= (0xffffffffUL ^ IN_MACROR_FLG)
+
 #define	res_single_shift sshift_condition = 0UL
 #define	res_right_locking_shift shift_condition &= 0xffffff0fUL
 #define	res_all_shift shift_condition &= 0xf0000000UL
-#define	res_left_shift shift_condition &= 0xf00000f0UL
-#define	res_right_shift shift_condition &= 0xf000000fUL
+#define	res_left_shift shift_condition &= 0xf00f00f0UL
+#define	res_right_shift shift_condition &= 0xf00f000fUL
 #define	res_locking_shift shift_condition &= 0xf000ff00UL
+#define	res_alt_shift shift_condition &= 0xf0000fffUL
 
 void shift_cond_recovery();
+
 /* --------------------------------------------------------------- */
 /* various utilities elsewhere 					   */
 /* --------------------------------------------------------------- */
 extern void	keis_conv P_((int, int));
 extern void	jef_conv P_((int, int));
+#if 0
 extern void	is_sbroken P_((int, int));
+#endif
 
 extern int	x0201conv P_((int, int));
 extern unsigned short x0201rconv P_((skf_ucode));
 
 extern int	latin2html P_((skf_ucode));
 extern int	latin2tex P_((skf_ucode));
+#if 0
 extern int	latin2htmlcnt P_((skf_ucode));
 extern int	latin2texcnt P_((skf_ucode));
+#endif
 
 extern int	is_in_encoded P_(());
 
+extern int	arib_macro_process P_((skfFILE *));
+extern int	arib_macro_rawproc P_((int *,int,int));
+extern int	arib_rpc_process P_((skfFILE *));
+
 extern void nec_g0_kanjiset();	/* set x-0208 kanji to g0	   */
 extern void nec_g0_kanaset();  	/* set x-0201 kana to g0	   */
 
@@ -475,8 +511,12 @@ extern short	in_keis;	/* KEIS83 mode in		   */
 /* --------------------------------------------------------------- */
 /* table of tables						   */
 /* --------------------------------------------------------------- */
+#ifndef FAST_MULT
 extern int   mul94[];
+#endif
+#if 0
 extern int   is_octet;		/* '1' means octet code set	   */
+#endif
 
 extern int   low_dbyte;		/* lower side double byte(left)	   */
 extern int   up_dbyte;		/* upper side double byte(right)   */
@@ -512,6 +552,7 @@ extern unsigned short uni_k_ibm_fa[];
 
 extern const char dakuten[];
 extern const int x213_sjis_map[];
+extern unsigned short x201_uni_byte[];
 
 extern unsigned short ebcdik40[];	/* EBCDIK Hitachi	   */
 extern unsigned short ebcdic40[];	/* EBCDIC IBM		   */
@@ -524,8 +565,8 @@ extern unsigned short ebcdic44[];	/* EBCDIC IBM037	   */
 
 extern unsigned short iscii_attrib_table[];
 
-#define up_has_tbl  (((up_dbyte > 1) && (up_ltable != NULL)) || \
-		     ((up_dbyte <= 1) && (up_table != NULL)))
+#define up_has_tbl  ((is_tbllong(up_dbyte) && (up_ltable != NULL)) || \
+		     (is_tblshort(up_dbyte) && (up_table != NULL)))
 
 extern int is_g0_table_x0201 P_(());
 
@@ -541,6 +582,8 @@ extern int in_codeset_preload();
 extern unsigned short *input_get_dummy_table();
 extern skf_ucode *input_get_dummy_ltable();
 extern int test_primary_codeset();
+extern int paraphrase_arib_macro P_((int));
+extern int is_charset_macro P_((struct iso_byte_defs *));
 
 extern struct iso_byte_defs *get_jisx0213_1_table();
 
@@ -585,19 +628,26 @@ extern char	*skf_table_lmagic;
 /* Unic*de decomposition/composition table definitions		   */
 /* --------------------------------------------------------------- */
 /* #define UNI_NKDL_TBL_LEN 13376 :* 0x00a0-0x33ff,0xf900-0xffff   */
-#define UNI_NKDL_TBL_LEN 17280	/* 0x00a0-0x33ff,0xf900-0xffff     */
+#define UNI_NKDL_TBL_LEN 20480	/* 0x00a0-0x33ff,0xf900-0xffff     */
 				/* 0x1d100-0x1d7ff,0x2f800-0x2fa20 */
+				/* KAITHI, ARAM, ESCP etc.	   */
+
+#define UNI_NORM_TBL_LEN 20480	/* about (UNI_TAIL_OFF) * 1.1	   */
 
-#define UNI_NORM_TBL_LEN 18600	/* (Unicode 5.0 = 16849) * 1.1	   */
 #define UNI_LAT_OFF	160	/* 0x00a0			   */
 #define UNI_UP_OFF	13152	/* 0x33ff - 0x00a0		   */
-#define UNI_COM_OFF	14944	/* UP_OFF + (0xf800 - 0xffff)	   */
-#define UNI_CJKC_OFF	16736	/* COM_OFF + (0x1d7ff - 0x1d100)   */
+#define UNI_COM_OFF	14944	/* UP_OFF + (0xf900 - 0xffff)	   */
+#define UNI_CJKC_OFF	16736	/* COM_OFF + (0x1d100 - 0x1d7ff)   */
+#define UNI_KAITHI_OFF	17280	/* CJKC_OFF + (0x2f800 - 0x2fa1f)  */
+#define UNI_ARAM_OFF	18672	/* KAITHI_OFF + (0x11090 - 0x11600)*/
+#define UNI_ECSP_OFF	18928	/* ARAM_OFF + (0x1ee00 - 0x1ef00)  */
+#define UNI_TAIL_OFF	19696	/* ECSP_OFF + (0x1f100 - 0x1f400)  */
 #define UNI_ATTRIB_TBL_LEN 65536
 
 extern unsigned short	*nkdc_lowptr;	/* area mapper		   */
 extern skf_ucode	*nkdc_lowmap;	/* map char table (in BSP) */
 extern unsigned short	*nkd_strmap;	/* map for compose strength*/
+extern int 		nkdc_lowsize;	/* map char table length   */
 
 #define UNIHAN_SBASE	0xac00	/* Hangul start point		   */
 #define UNIHAN_LBASE	0x1100
@@ -607,11 +657,16 @@ extern unsigned short	*nkd_strmap;	/* map for compose strength*/
 #define UNIHAN_VCOUNT	21
 #define UNIHAN_TCOUNT	28
 
+#define ARIB_MACRO_LIMIT	128
+#define ARIB_RPC_MGN	6
+
 /* --------------------------------------------------------------- */
 #if defined(ROT_SUPPORT) && defined(NEW_ROT_CODE)
 extern int skf_rot47conv_d P_((int));
 extern int skf_rot13conv_d P_((int));
 #endif
+
+extern void dump_table_address P_((struct iso_byte_defs *,char *));
 /* --------------------------------------------------------------- */
 /* internationalized IDN decoders				   */
 /* --------------------------------------------------------------- */
@@ -655,6 +710,7 @@ extern int skf_rot13conv_d P_((int));
 #define punycode_unexpeof	-7
 
 extern skf_ucode puny_adapt P_ ((long, long, int));
+extern int	is_prohibit P_ ((skf_ucode));
 extern int	in_ace;
 #endif
 
