@@ -22,7 +22,7 @@
 **	arising by using whole or a part of this software.
 **********************************************************************
 ** error.c:	error message related routine in skf
-** $Id: error.c,v 1.139 2015/05/23 14:30:12 seiji Exp seiji $
+** $Id: error.c,v 1.141 2016/06/14 14:56:32 seiji Exp seiji $
 **	out_area_out:	error character warning generation
 **	bogus_out:	same as above
 **
@@ -41,6 +41,7 @@
 #include <errno.h>
 #include "convert.h"
 #include "oconv.h"
+#include "in_code_table.h"
 
 #if HAVE_LOCALE_H
 #include <locale.h>
@@ -297,12 +298,12 @@ skf_gettext("skf: this code point(u+%04x) output is prohibited\n");
 #ifndef	UNDEF_NOCLASH
     if ((ucode_undef != 0) && !(is_o_encode) && (c1 >= 0) && (donotout == 0)) {
 	    post_oconv(ucode_undef);
-    } else if (c1 >= 0) {
+    } else if ((c1 >= 0) && (donotout == 0)) {
 	    post_oconv('.');
 	    post_oconv('.');
     } else;
 #else
-    if (c1 >= 0) {
+    if ((c1 >= 0) && (donotout == 0)) {
 	    post_oconv('.');
 	    post_oconv('.');
     } else;
@@ -557,14 +558,17 @@ int mode;
 #ifdef ENABLE_DISABLED
     fprintf(skfstderr,"ED ");
 #endif
+#ifdef HAVE_MKSTEMP
+    fprintf(skfstderr,"OMST ");
+#endif
 #ifdef SKF_EXPERIMENTAL
     fprintf(skfstderr,"EXP ");
 #endif
     (void)fputc('\n',skfstderr);
   } else;
 
-    skf_err_msg = skf_gettext("FEATURES: ");
-    fprintf(skfstderr,skf_err_msg);
+  skf_err_msg = skf_gettext("FEATURES: ");
+  fprintf(skfstderr,skf_err_msg);
 #ifdef KUNIMOTO
     fprintf(skfstderr,"KN ");
 #endif
@@ -665,9 +669,9 @@ void display_nkf_help()
     fprintf(skfstderr," ----------------- ----------------------------------------------\n");
     fprintf(skfstderr," -w                -z\n");               
     fprintf(skfstderr," -W                -Z\n");               
-    fprintf(skfstderr," -w16L             --oc=utf-16le\n");               
+    fprintf(skfstderr," -w16L             --oc=utf-16le-bom\n");
     fprintf(skfstderr," -W16L             --ic=utf-16le\n");               
-    fprintf(skfstderr," -w32L             --oc=utf-32le\n");               
+    fprintf(skfstderr," -w32L             --oc=utf-32le-bom\n"); 
     fprintf(skfstderr," -W32L             --ic=utf-32le\n");               
     fprintf(skfstderr," -Z0               --enable-ascii-conv\n");
     fprintf(skfstderr," -Z1               --enable-ascii-conv --enable-space-conv\n");
@@ -678,9 +682,9 @@ void display_nkf_help()
     fprintf(skfstderr," --cap-input       --decode=cap\n");
     fprintf(skfstderr," --url-input       --decode=url\n");
     fprintf(skfstderr," --numchar-input   --decode=hex\n");
-    fprintf(skfstderr," --guess=1         -g\n");
+    fprintf(skfstderr," --guess=1         --inquiry\n");
     fprintf(skfstderr," --guess=2         --hard-inquiry\n");
-    fprintf(skfstderr," --ic=utf-8-bom    -z --enable-endian-mark\n");
+    fprintf(skfstderr," --oc=utf-8-bom    -z --enable-endian-mark\n");
     fprintf(skfstderr," ----------------- ----------------------------------------------\n");
     skf_err_msg = skf_gettext(
     	" * following options are not supported: -i -o -T --prefix --fb-skip \n");
@@ -836,19 +840,23 @@ long	a1,a2;
 		g3_table_mod->desc);
 	fprintf(skfstderr,"\n low_table:%08lx\n",(unsigned long)low_table);
 	fprintf(skfstderr," up_table:%08lx\n",(unsigned long)up_table);
-    } else if (code >= SKF_TABLEERR_G0) {
+    } else if (code >= SKF_TABLEERR_G0A) {
       switch(code) {
 	case SKF_TABLEERR_G0A:
 	    skf_err_msg = skf_gettext(
 		"Generic g%1dalt table loading error (table: %s)\n");
 	    tdefs = gx_table_mod; code++; break;
 	case SKF_TABLEERR_G0:
+	    tdefs = g0_table_mod; 	/*@FALLTHROUGH@*/
 	case SKF_TABLEERR_G1:
+	    if (tdefs == NULL) tdefs = g1_table_mod; /*@FALLTHROUGH@*/
 	case SKF_TABLEERR_G2:
+	    if (tdefs == NULL) tdefs = g2_table_mod; /*@FALLTHROUGH@*/
 	case SKF_TABLEERR_G3:
 	    skf_err_msg = skf_gettext(
 		"Generic g%1d table loading error (table: %s)\n");
-	    tdefs = g0_table_mod; break;
+	    if (tdefs == NULL) tdefs = g3_table_mod;
+	    break;
 	default:
 	    skf_err_msg = skf_gettext(
 	    "unassigned error(%s)\n");
@@ -1049,15 +1057,13 @@ void	skf_outcode_display()
 {
 /* name and language of the codeset */
     if ((out_codeset > 0) && (out_codeset < codeset_end)) {
-	fprintf(skfstderr,"%s (#%d,%x%x,typ:%lx) ",
+    	out_codeset = DEFAULT_O;
+    } else;
+    fprintf(skfstderr,"%s (#%d,%x%x,typ:%lx) ",
 	    i_codeset[out_codeset].desc,out_codeset,
 	    ((i_codeset[out_codeset].oconv_lang) >> 8) & 0x7f,
 	    (i_codeset[out_codeset].oconv_lang) & 0x7f,
 	    i_codeset[out_codeset].oconv_type);
-    } else {
-	skf_err_msg = skf_gettext("Unknown(internal error)");
-	(void)fputs(skf_err_msg,skfstderr);
-    };
     (void)fflush(skfstderr);
 }
 /* --------------------------------------------------------------- */
@@ -1076,6 +1082,7 @@ void test_support_charset()
 
     (void)fflush(skfstderr); (void)fflush(stdout);
     for (i=0; iso_ubytedef_table[i].ientry != NULL; i++) { 
+    	if ((i == ub_ovlay_index) || (i == prv_ovlay_index)) continue;
 	fprintf(skfstderr,"# %s:\n",iso_ubytedef_table[i].desc);
 	uentry = iso_ubytedef_table[i].ientry;
 	for (j=0; uentry[j].defschar != 0; j++) {
@@ -1100,7 +1107,7 @@ void test_support_charset()
 	      } else;
 	    } else {
 #if defined(DYNAMIC_LOADING) && !defined(SWIG_EXT)
-		if (load_external_table(&(uentry[i])) == 0) {
+		if (load_external_table(&(uentry[j])) == 0) {
 		  if (uentry[j].desc != NULL)
 		    fprintf(skfstderr,"%s%s%s*\n",
 		    skf_cname,pad,uentry[j].desc);
