@@ -64,9 +64,6 @@ int esc_process(f)	/* escape parsing process.		   */
     int idx;		/* table search index			   */
     struct iso_byte_defs *g_table_mod;
 
-#ifdef SKFDEBUG
-    if (is_vv_debug) fprintf(stderr,"-E-");
-#endif
     if ((c1 = vGETC(f)) == sEOF) {  /* case: esc-eof		   */
 		;		/* pass esc may screw up terminal  */
 /* --------------------------------------------------------------- */
@@ -648,6 +645,7 @@ int esc_process(f)	/* escape parsing process.		   */
 	    } else {
 		res_all_shift;
 	    };
+	    codeset_stack_depth = 0;
 	} else if (c1 == 0x47) { /* to UTF-8 with return	   */
 	    if (codeset_stack_depth == 0) {
 		in_codeset_sr_save = in_codeset;
@@ -661,13 +659,13 @@ int esc_process(f)	/* escape parsing process.		   */
 	    } else {
 		res_all_shift;
 	    };
-	} else if (c1 == '?') { /* X 0221(1994). extensive test	   */
+	} else if (c1 == '/') { /* X 0221(1994). extensive test	   */
 		/* Since we could not treat any composite code in  */
 		/* *Code converter*. I regard whole level as same  */
 	    if (((c3 = vGETC (f)) == 0)) { /* padding  */
 		while (skip_x221_2_null() == 0) { c3 = vGETC(f); };
 	    };
-	    if (c1 == sEOF) {	/* case: esc-%-?-eof		   */
+	    if (c3 == sEOF) {	/* case: esc-%-?-eof		   */
 		in_undefined('u',SKF_UNEXPEOF); /* Do nothing	   */
 	    } else if (c3 == '@') {	/* 2Octet Level-1 ISO10646 */
 		in_codeset = codeset_utf16be; /* point of no-return   */
@@ -709,6 +707,38 @@ int esc_process(f)	/* escape parsing process.		   */
 		fprintf(stderr," 2022-extcode: %2x,%2x,%2x",'%',c1,c3);
 #endif
 /* --------------------------------------------------------------- */
+/* Coding system change(2) : JIS X-0221 use this sequence	   */
+/* --------------------------------------------------------------- */
+    } else if ((c1 == '%') && (coding_system_sense)) {
+				/* code extension method select	   */
+	if ((c1 = vGETC(f)) == sEOF) {	/* case: esc-2/5-eof	   */
+	    in_undefined('x',SKF_UNEXPEOF);
+	    res_all_shift;
+	} else if (c1 == '@') { /* standard return. do nothing	   */
+#ifdef SKFDEBUG
+	    if (is_vv_debug)
+		fprintf(stderr," codesw: %c,%2x - ",'%',c1);
+#endif
+	    if (codeset_stack_depth > 0) { /*@-globstate@*/
+		in_codeset = in_codeset_sr_save;
+		encode_cap = encode_cap_sr_save;
+		ucod_flavor = ucod_fl_sr_save;
+		codeset_flavor = codeset_fl_sr_save;
+		clear_after_mime();
+		return(sOCD);
+	    } else {
+		in_undefined('%',SKF_UNSUPP);
+		res_all_shift;
+	    };
+	} else { /* only coding system return is detected	   */
+#ifdef SKFDEBUG
+	    if (is_vv_debug)
+		fprintf(stderr," codesw: %c,%2x - ",'%',c1);
+#endif
+	    in_undefined('%',SKF_UNSUPP);
+	    res_all_shift;
+	};
+/* --------------------------------------------------------------- */
 /* Miscellaneous ISO-2022 sequences				   */
 /* --------------------------------------------------------------- */
     } else if (c1 == 0x21) {    /* control character call sequence */
@@ -721,10 +751,10 @@ int esc_process(f)	/* escape parsing process.		   */
 	    c1 = seq_sweep(f,FALSE);	/* bite it from stream	   */
 	} else  ;
     } else if (c1 == '&') {     /* version announce sequence	   */
-	if (c1 == sEOF) {		/* case: esc-&-eof         */
+	if ((c3 = vGETC(f)) == sEOF) {	/* case: esc-&-eof         */
 	    in_undefined('a',SKF_UNEXPEOF);
 	    res_all_shift;	/* just discard.		   */
-	} else if (c1 >= 'A') { /* X 0208(1990) or later. 	   */
+	} else if (c3 >= 'A') { /* X 0208(1990) or later. 	   */
 		/* we already assume 1990 revision, so ignore it.  */
 	    ; 			/* forget unknown one ;^)	   */
 	};
@@ -761,7 +791,7 @@ int esc_process(f)	/* escape parsing process.		   */
 #endif
 	res_single_shift; 
 	if (is_euc_cns(i_codeset[in_codeset].encode)) {
-	    if ((c1 = vGETC(f)) < 0) {	/* valid codeset	   */
+	    if ((c1 = vGETC(f)) > A_SP) {  /* valid codeset	   */
 		if ((c1 >= 0xa1) && (c1 <= 0xa0 + cns11643_defined_code)) {
 		    set_ss2;
 		    g2_table_mod = 
@@ -890,7 +920,8 @@ int	out;		/* out: output character if TRUE	   */
 	    /* --- note: actual termination char is > 0x30, but	   */
 	    /* 0x30-0x3f is reserved for private purpose, and not  */
 	    /* likely to be contained in standard network message. */
-	    	if (out) ox_ascii_conv(c1); break;
+	    	if (out) ox_ascii_conv(c1);
+		break;
 	    } else if (c1 < A_SP) {	/* or in area never happen */
 	    	/* character drop or input is corrupted		   */
 	    	if (c1 == A_ESC) {
