@@ -62,10 +62,29 @@ void @gen@_cjk_oconv P_((skf_ucode));
 void SKF@gen@STROUT(st)
 const char *st;
 {
+#if	defined(SKFKEISSW)
+    int len,sy,c3;
+    for (len=0; (len<FIXSTRLEN_LIMIT) && (*st != '\0') ; len++,st++) {
+    	sy = *(st);
+	c3 = uni_o_ascii[sy];
+	if (c3 >= 0x100) {
+	    SKFKEISOUT(c3);
+	} else {
+	    if (c3 != 0) {
+		SKFKEIS1OUT(c3);
+	    } else {
+		if (sy < A_SP) {
+		    SKFKEIS1OUT(sy); 
+		} else;	/* DUNNO				   */
+	    };
+	};
+    };
+#else /* NOT KEIS */
     int len;
-    for (len=0; (len<24) && (*st != '\0') ; len++,st++) {
+    for (len=0; (len<30) && (*st != '\0') ; len++,st++) {
 	SKF@gen@1OUT(*st);
     };
+#endif
 }
 #endif
 /* --- incantinate all macros ... -------------------------------- */
@@ -147,6 +166,11 @@ void @gen@_ascii_oconv (c1)
 	};
 #if	defined(SKFJISSW)
     } else if (is_2ln(c3)) {
+    	if (is_3pl(c3)) {
+	    SKF@gen@1OUT(c3);
+	} else SKF@gen@8859OUT(c3);
+	return;
+    } else if (is_4pl(c3)) {
     	SKF@gen@8859OUT(c3);
 	return;
 #endif
@@ -249,10 +273,14 @@ void @gen@_latin_oconv (c4)
 		SKF@gen@1OUT(c3);
 		converted = 1;
 	    };
-#if !defined(SKFSJISSW)
+#if defined(SKFJISSW) 
 	} else if (is_2ln(c3)) {  /* always some kind of iso8859    */
-	    SKFJIS8859OUT(c3);
+	    if (is_3pl(c3)) {
+		SKF@gen@1OUT(c3);
+	    } else SKF@gen@8859OUT(c3);
 	    converted = 1;
+#endif
+#if !defined(SKFSJISSW)
 	} else if (is_2pl(c3) &&
 		(use_x0212 || (!is_jis(conv_cap) && !is_euc(conv_cap)))) {
 	    SKF@gen@G3OUT(c3);
@@ -263,9 +291,11 @@ void @gen@_latin_oconv (c4)
 #else
 	} else if (is_2pl(c3) && 
 			(is_ms_213c(conv_cap) || is_ms_cel(conv_cap))) {
+#ifdef SKFDEBUG
 	    if (is_vv_debug) {
 		fprintf(stderr," 2P");
 	    } else;
+#endif
 	    SKF@gen@G3OUT(c3);
 	    converted = 1;
 #endif	/* SKFSJISSW */
@@ -285,12 +315,14 @@ void @gen@_latin_oconv (c4)
 		converted = latin2tex(c4);
 	};
     };
-#ifdef SKFSJISSW
-#endif
     if (converted == 0) {
+#ifdef SKFSJISSW
 	if (out_codeset == codeset_cp932w) {
 	    out_undefined(c4,SKF_OUNDEF);
 	} else if (c4 < 0x100) {
+#else
+	if (c4 < 0x100) {
+#endif
 	    ascii_fract_conv(c1);
 #if defined (SKFKEISSW)
 	} else if (is_keis_jef(conv_cap) && ((c1 == 0xd2) || (c1 == 0xd4))) {
@@ -327,7 +359,7 @@ void @gen@_cjkkana_oconv (c2)
 #if !defined(SKFKEISSW)
 	if (o_encode) out_@gen@_encode(c2,c2);
 #endif
-	if (sup_space_conv) {
+	if (sup_space_conv && (uni_o_kana != NULL)) {
 	    ch = uni_o_kana[c1];
 	    SKF@gen@OUT(ch);
 	} else {
@@ -381,7 +413,9 @@ void @gen@_cjkkana_oconv (c2)
 		};
 #if	defined(SKFJISSW)
 	    } else if (is_2ln(ch)) {
-		SKF@gen@8859OUT(ch);
+		if (is_3pl(ch)) {
+		    SKF@gen@1OUT(ch);
+		} else SKF@gen@8859OUT(ch);
 		return;
 #endif
 #if !defined(SKFSJISSW)
@@ -459,8 +493,10 @@ void @gen@_cjk_oconv (c3)
 	    } else;
 #if	defined(SKFJISSW)
 	} else if (is_2ln(c4)) {
-		SKF@gen@8859OUT(c4);
-		return;
+	    if (is_3pl(c4)) {
+		SKF@gen@1OUT(c4);
+	    } else SKF@gen@8859OUT(c4);
+	    return;
 #endif
 	} else if ((c4 != 0) && (c4 < 0x80)) {
 	    SKF@gen@1OUT(c4); return;
@@ -503,6 +539,18 @@ void @gen@_ozone_oconv(c2)
     if ((c2 >= HNGL_TBL_START) && (c2 <= HNGL_TBL_END)) {/* Hangles */
 	if (uni_o_hngl != NULL) {
 	    ch = uni_o_hngl[c2 - HNGL_TBL_START];
+#if defined(SKFJISSW)
+	    if (ch >= 0x100) {
+		if (!is_hangul_shift) {	/* rfc1554 */
+		    SKFputc(A_ESC);SKFputc(ag0_mid);SKFputc(ag0_midl);
+		    SKFputc('C');
+		    set_hangul_shift;
+		};
+		SKFputc((ch >> 8) & 0x7f); SKFputc(ch & 0x7f);
+	    } else if (ch > 0)  {
+		SKF@gen@1OUT(ch);
+	    };
+#endif	/* SKFJISSW */
 #if defined(SKFBGSW)
 	} else if (is_gb18030(conv_cap)) {
 	    ch = c2 - 0x5543;	/* 0x82 35 98 03 = 01 05 17 03	   */
@@ -604,17 +652,18 @@ void @gen@_ozone_oconv(c2)
 	    };
 #if	defined(SKFJISSW)
 	} else if (is_2ln(ch)) {
-	    if (is_vv_debug) {
-		fprintf(stderr,"2L");
-	    } else;
-	    SKF@gen@8859OUT(ch);
+	    if (is_3pl(ch)) {
+		SKF@gen@1OUT(ch);
+	    } else SKF@gen@8859OUT(ch);
 	    return;
 #endif
 #if !defined(SKFSJISSW)
 	} else if (is_2pl(ch) && use_x0212) {
+#ifdef SKFDEBUG
 	    if (is_vv_debug) {
 		fprintf(stderr,"2P");
 	    } else;
+#endif
 	    SKF@gen@G3OUT(ch);
 	    converted = 1;
 	} else if (is_3pl(ch)) {
@@ -716,7 +765,9 @@ void @gen@_compat_oconv (c2)
 	    if (c3 >= 0x8000) {	/* dim2 & 4		   */
 #if	defined(SKFJISSW)
 		if (is_2ln(c3)) {
-		    SKF@gen@8859OUT(c3);
+		    if (is_3pl(c3)) {
+			SKF@gen@1OUT(c3);
+		    } else SKF@gen@8859OUT(c3);
 		    return;
 		} else 
 #endif
@@ -899,11 +950,13 @@ void @gen@_private_oconv (c2)	/* u+d800 to u+f8ff		   */
 		   || ((c2 >= 0xe6b1) && (c2 <= 0xe6ba))
 		   || ((c2 >= 0xe6d0) && (c2 <= 0xe70b))
 		   || ((c2 >= 0xe70c) && (c2 <= 0xe757))) {
-		    ch = uni_o_prv[c2 - 0xe000];
-		    if (ch != 0) {
-			c1 = (ch >> 8) & 0xff;
-			SKFputc(c1); SKFputc(ch & 0xff);
-			return;
+		    if (uni_o_prv != NULL) {
+			ch = uni_o_prv[c2 - 0xe000];
+			if (ch != 0) {
+			    c1 = (ch >> 8) & 0xff;
+			    SKFputc(c1); SKFputc(ch & 0xff);
+			    return;
+			};
 		    };
 		};
 	    };
@@ -1180,18 +1233,10 @@ skf_ucode convch;	/* table lookup result */
 		    if (add_renew) {
 		    	plen += 2; slen++;
 		    } else;
-		    if (is_euc7(conv_cap)) {
-			slen++; plen += 2;
-		    } else {
-		    	if (ag0_quad) plen += (4+3);
-			else plen += (3+3);
-		    };
+		    if (ag0_quad) plen += (4+3);
+		    else plen += (3+3);
 		} else {
-		    if (is_euc7(conv_cap)) {
-		    	plen += 1;
-		    } else {
-			plen += 3;	
-		    };
+		    plen += 3;	
 		};
 #endif
 #ifdef SKFEUCSW
