@@ -156,6 +156,11 @@ skfFILE *f;			/* can process euc, euc+jis-mix	   */
     encode = i_codeset[in_codeset].encode;
     if (is_nkf_compat && ((in_codeset == codeset_eucjp)
 			|| (in_codeset == codeset_jis)
+			|| (in_codeset == codeset_x0213)
+			|| (in_codeset == codeset_x213a)
+			|| (in_codeset == codeset_x0213_s)
+			|| (in_codeset == codeset_euc_213)
+			|| (in_codeset == codeset_euc_213a)
 			|| (in_codeset == codeset_cp5022x)
 			|| (in_codeset == codeset_cp50221)
 			|| (in_codeset == codeset_cp50222)
@@ -172,6 +177,13 @@ skfFILE *f;			/* can process euc, euc+jis-mix	   */
 	return(be_in(f));
     } else;
 
+    /* wrong state change guard */
+    if (low_table == NULL) {
+    	if (g0_table_mod == NULL)
+		g0_table_mod = &(iso_unibyte_defs[ascii_index]);
+	g0table2low();
+    } else;
+
     /* --- conversion loop --------------------------------------- */
     while (TRUE) {
 	if (c2) {       /* ***** second byte ********************* */
@@ -295,7 +307,7 @@ skfFILE *f;			/* can process euc, euc+jis-mix	   */
 			    ((c2s == 0xf4) &&
 			     ((c1s < 0xa1) || (c1s > 0xa6)))) {
 		    /* cp932 - cp51932 conversion bug recovery */
-			k0 = 0xe000 + ((c2 - 0xf0) * 188) 
+			k0 = 0xe000 + ((c2s - 0xf0) * 188) 
 			    + (c1 - 0x40) + ((c1 < A_DEL) ? 0 : 1);
 		    } else;
 		} else if (c2s > A_KSP) {
@@ -667,6 +679,13 @@ skfFILE *f;			/* can process sjis, big5 etc.	   */
 	return(be_in(f));
     } else;
 
+    /* wrong state change guard */
+    if (low_table == NULL) {
+    	if (g0_table_mod == NULL)
+		g0_table_mod = &(iso_unibyte_defs[ascii_index]);
+	g0table2low();
+    } else;
+
     while (TRUE) {
 	if (c2) {       /* ***** second byte ********************* */
 	    c2s = c2;	/* save for later test			   */
@@ -1178,10 +1197,11 @@ skf_ucode ch;
 int	cod;
 {
     skf_ucode c4,c5;
+    int	c1;
 
     if ((ch == 0xfeff) || (ch == 0xfffe)) { /* BOM		  */
 	return(0);		/* just discard			  */
-    } if ((ch >= 0xd800) && (ch <=0xdbff)){ /* surrogate	  */
+    } else if ((ch >= 0xd800) && (ch <=0xdbff)){ /* surrogate	  */
 		    /* utf7 with surrogate case		  */
 	if ((c4 = u_dec_hook(f,cod)) == sEOF) {
 	    in_undefined(c4, SKF_NOSURRG); 
@@ -1193,6 +1213,20 @@ int	cod;
 	c5 = ((ch - 0xd800) << 10) + ((c4 - 0xdc00) & 0x3ff)
 	    + 0x10000;
 	oconv(c5);
+    } else if (ch == A_ESC) {
+	if ((c4 = u_dec_hook(f,cod)) == sEOF) {
+	    in_undefined(c4, SKF_UNEXPEOF); 
+	    return(sEOF);
+	} else if ((c5 = u_dec_hook(f,cod)) == sEOF) {
+	    in_undefined(c4, SKF_UNEXPEOF); 
+	    return(sEOF);
+	} else;
+	if ((c4 == '%') && (c5 == '@')) { /* standard return	   */
+	    enque(c4); enque(c5);
+	    if ((c1 = esc_process(f)) < 0) return(c1);
+	} else {
+	    oconv(A_ESC); oconv(c4); oconv(c5);
+	};
     } else if ((ch < 0x110000) || unchk_utf32_range) {
 	oconv(ch);
     } else {
@@ -1305,6 +1339,11 @@ register skfFILE *f;
 {
     int c1;
 
+    if (is_mimeb_encoded || is_mimeq_encoded || is_mimeb_strict 
+    	|| is_puny_encoded) {
+	reset_encoded;
+    } else;
+
     while (TRUE) {
 	if ((c1 = vGETC(f)) == sEOF) {
 	    return(sEOF);
@@ -1500,7 +1539,10 @@ unsigned long ec;	/* encode				  */
     };
 
     if (is_vni(ec)) {
-	if ((ca < 0xc0) || ((cy = viq_tbl[ca - 0x40]) == 0)) {
+	if (ca < 0x40) {
+	    in_sbroken(ch,ca);
+	    return(-1);
+	} else if ((ca < 0xc0) || ((cy = viq_tbl[ca - 0x40]) == 0)) {
 	    oconv(ch); return(ca);
 	};
 	/* ch other than follows never comes here */
@@ -1998,20 +2040,26 @@ skfFILE *f;			/* can process euc, euc+jis-mix	   */
     skf_input_lang = M_JP;
 
     encode = i_codeset[in_codeset].encode;
+    g1table2up();
 #if defined(ROT_SUPPORT) && defined(NEW_ROT_CODE) 
     if (nkf_compat && is_nkf_rotmode) mod = TRUE;
     else mod = FALSE;
 #endif
 
+    /* wrong state change guard */
+    if (low_table == NULL) {
+    	if (g0_table_mod == NULL)
+		g0_table_mod = &(iso_unibyte_defs[ascii_index]);
+	g0table2low();
+    } else;
+
     /* --- conversion loop --------------------------------------- */
     while (TRUE) {
 	if (c2) {       /* ***** second byte ********************* */
 	    c2s = c2;	/* save for later test			   */
 	    if ((c1 = vGETC(f)) == sEOF) {
 			/* c2 is rot'd even if c1 is EOF	   */
-		if (!encode_enbl || !is_mime_nkfmode) {
-			in_undefined(c2,SKF_UNEXPEOF);
-		};
+		in_undefined(c2,SKF_UNEXPEOF);
 		break;
 	    } else if (c1 == sOCD) return(sOCD);
 	    c1s = c1;
@@ -2020,7 +2068,7 @@ skfFILE *f;			/* can process euc, euc+jis-mix	   */
 #endif
 	    c3 = c2;
 	    if (is_msfam(encode)) {
-	      if ((c1 <= 0xfc) && (c1 >= 0x40)
+	      if ((c1 <= 0xff) && (c1 >= 0x21)
 			&& (c1 != 0x7f)) {
 		if ((c2 <= 0xef) || (is_ms_213c(encode) == 0)) {
 					/* can convert into JIS	   */
@@ -2127,16 +2175,18 @@ skfFILE *f;			/* can process euc, euc+jis-mix	   */
 		};
 	    } else if (((c2s < A_DEL) && (ch < low_table_limit))
 		     || ((c2s > A_KSP) && (ch <up_table_limit))) {
-		if (is_euc_mseuc(encode) || is_cp5022x(encode)) {
+		if (is_cp5022x(encode)) {
 		    if (ch < X0208_KANJI_END) {
 			if (is_cp5022x(encode)) k0 = low_table[ch];
 			else k0 = up_table[ch];
+#if 0
 		    } else if ((((c2 >= 0x75) && (c2 < 0x79))
 			    || ((c2 >= 0x7d) && (c2 <= 0x7e)))
 			    && (c1 > A_SP)
 			    && (is_euc_mseuc(encode))) {
 			k0 = ((c2s - 0xf5) * 94)
 			    + (c1s - 0xa1) + 0xe40c;
+#endif
 		    } else if ((c2 < 0x7d) &&
 			(((c1 > A_SP) && (is_cp5022x(encode))) ||
 			(((c1s > A_KSP)
@@ -2251,7 +2301,14 @@ skfFILE *f;			/* can process euc, euc+jis-mix	   */
 		/* --- 0xa1 <= c1 <= 0xff ------------------------ */
 	    } else if (c1 >= A_KSP) {	/* c1 >= 0xa0		   */
 			/* code is within JIS upper plane	   */
-		if (up_dbyte) {
+		if (is_msfam(encode)) {
+		    if (c1 <= KANA_END) {
+			oconv(c1 + 0xff60 - 0xa0);
+	    	    } else { /* 0x80 <= c1 < 0xa0 && is_msfam	*/
+			c2 = c1; continue;
+		    };
+		} else if (is_multibyte(up_dbyte) ||
+		    ((c1 > KANA_END) && is_msfam(encode))) {
 				/* lower plane is 2byte locked	   */
 		    c2 = c1 ;	/* pass it to second byte	   */
 		    continue;	/* goto next_byte		   */
@@ -2266,9 +2323,13 @@ skfFILE *f;			/* can process euc, euc+jis-mix	   */
 		};
 		/* --- rest 0x80 <= c1 < 0xa0 -------------------- */
 		/* comparison is for KOI-8-R consideration	   */
+	    } else if (is_msfam(encode)) {
+		c2 = c1; continue;
+#if 0
 	    } else if ((c1_table_mod != NULL) && 
 	    		(c1_table_mod->unitbl != NULL)) {
 		if ((c1 = c1_process(f,c1)) < 0) return(c1);
+#endif
 	    } else if (is_cp5022x(encode) && (c1 <= 0x98)) {
 		c2 = c1; continue;
 	    } else {	/* 0x80 <= c1 < 0xa0 & !is_msfam	   */
