@@ -391,13 +391,13 @@ int racecode_decode(input_length, input, output_length, output)
 	    if ((uw = race_decode_digit(input[i++])) < 0) {
 		return punycode_bad_input; /* should not occur	   */
 	    };
-	    chu = res + ((uv & 0x1fU) << 2) + ((uw & 0x18U) >> 3);
-	    res = ((uw & 0x07U) << 5);
-	    b32stat = 3;
+	    chu = res + uv;
+	    res = 0;
+	    b32stat = 0;
 	} else if (b32stat == 3) {
-	    chu = res + ((uv >> 2) & 0xfU); b32stat = 2;
-	    res = (uv << 6) & 0xc0U;
-	    b32stat = 3;
+	    chu = res + uv;
+	    res = 0;
+	    b32stat = 0;
 	} else {	/* b32stat == 0				   */
 	    if ((uw = race_decode_digit(input[i++])) < 0) {
 		return punycode_bad_input; /* should not occur	   */
@@ -409,8 +409,11 @@ int racecode_decode(input_length, input, output_length, output)
 	if (((i != 0) || (chu != 0xd8)) 
 		&& (lcheck == 0) && (wn1 == 0)) {
 	    u1 = chu; wn1 = 1;
+	    if (chu == 0xd8) {
+	    	wn1 = 8; lcheck = 1;
+	    } else;
 	} else if (wn1 == 1) {		/* Step 2		   */
-	    if (uv == 0xff) {
+	    if (chu == 0xff) {
 		wn1 = 5; continue;
 	    };
 	    if ((u1 == 0) && (chu == 0x99)) { /* Step 3 & 4	   */
@@ -420,7 +423,9 @@ int racecode_decode(input_length, input, output_length, output)
 	    *output_length += 1;
 	    wn1 = 1; continue; 
 	} else if (wn1 == 5) {		/* Step 5		   */
-	    if (chu == 0x99) {
+	    if ((i == (input_length - 1)) && (b32stat == 0)) {
+	    	return(punycode_bad_input);
+	    } else if (chu == 0x99) {
 		output[*output_length] = (u1 << 8) + 0xff;
 		*output_length += 1;
 	    } else {
@@ -435,6 +440,7 @@ int racecode_decode(input_length, input, output_length, output)
 	    *output_length += 1;
 	    lcheck = 1; continue;
 	} else {	/* LCHECK start  case			   */
+	    u1 = chu;
 	    lcheck = 1; continue; /* skip lcheck validate, for	   */
 			  /* such check in skf is worthless	   */
 	}; 
@@ -1123,17 +1129,8 @@ int	flg;
 		  if ((c3 == A_LF) && (c1 == A_CR)) continue;
 		  /* other case: restart at c3			   */
 		  pre_Qenque(c3);
-		  continue;
 	      };
-	      if (!is_nkf_compat || !is_hex_char(c1)) continue;
-	      if ((c3 = hook_q_getc(f,flg)) == sEOF) continue;
-#ifdef SKFDEBUG
-	      if (is_vv_debug) fprintf(stderr,"-%02x ",c3);
-#endif
-	      pre_Qenque(c1); pre_Qenque(c3);
-	      if (!is_hex_char(c3)) continue;   /* broken	   */
-	      ch = (skf_hex(c1) << 4) + skf_hex(c3);
-	      post_Qclear(); pre_Qclear(); return(ch);
+	      continue;
 	  };
 	} else if (in_mime && is_b_encode) {
 #ifdef SKFDEBUG
@@ -1842,9 +1839,9 @@ int cod;
 	  };
 	} else {		/* utf-16 (a.k.a. UCS2)		   */
 #ifdef SKFDEBUG
-	  if (is_vv_debug) {
+	    if (is_vv_debug) {
 	  	fprintf(stderr," U2:%d",in_codeset);
-	  } else;
+	    } else;
 #endif
 	    c2 = c1;
 	    GETC_add(c1,c2,f);
