@@ -51,9 +51,6 @@ static void SKFROTPUT P_((int));
 /* --------------------------------------------------------------- */
 /* conversion tables						   */
 /* --------------------------------------------------------------- */
-#define	   is_low_dbyte		(low_dbyte)
-#define	   is_up_dbyte		(up_dbyte)
-
 /* --- language tag save areas ----------------------------------- */
 static  int	lang_tag_index = 0;
 
@@ -303,23 +300,49 @@ int	oconv_init()
     if (is_o_encode) {
 /* encoding and MIME */
     /* do not set encoding if GL part is not iso-2022 like code */
-    	if (is_ucs_utf7(conv_cap) || is_ucs_brgt(conv_cap)
+    	if (is_ucs_utf7(conv_cap) || is_brgt(conv_cap)
 	  || is_ucs_uri(conv_cap)
 	  || is_ucs_transp(conv_cap)
+	  || (in_codeset == codeset_transp)
 	  || is_hzzW(conv_cap) || is_vni(conv_cap) 
 	  || is_viqr_or_vimn(conv_cap) 
 	  || is_keis(conv_cap)) {
 	  /* these output encodings do not support MIMEs	*/
 	  out_undefined(sFLSH,SKF_ENC_ERR);
 	  o_encode = 0;
+	} else if ((out_codeset == codeset_binary)
+	  || (out_codeset == codeset_nyukan_utf8)
+	  || (out_codeset == codeset_nyukan_utf16)
+	  || (out_codeset == codeset_locale)
+	  || (out_codeset == codeset_puny)
+	  || (out_codeset == codeset_transp)
+	  || (out_codeset == codeset_gsm0338)
+	  || (out_codeset == codeset_brgt)
+	  || (out_codeset == codeset_sjisau)
+	  || (out_codeset == codeset_sjissb)
+	  || (out_codeset == codeset_sjisontt)
+	  || (out_codeset == codeset_sjiscl)
+	  || (out_codeset == codeset_maccyrl)
+	  || (out_codeset == codeset_macCE)
+	  || (out_codeset == codeset_macroman)
+	  || (out_codeset == codeset_macdevang)
+	  || (out_codeset == codeset_macturka)
+	  || (out_codeset == codeset_macgreek)
+	  || (out_codeset == codeset_maciceln)
+	  || (out_codeset == codeset_macgujar)
+	  || (out_codeset == codeset_macgurmu)
+	  || (out_codeset == codeset_maccroatian)
+	  || (out_codeset == codeset_macromna)
+	  || (out_codeset == codeset_uri)
+	  || (out_codeset == codeset_kr)) {
+	  out_undefined(sFLSH,SKF_ENC_ERR);
+	  o_encode = 0;
 	} else;
 	if (is_o_encode_hex(o_encode) 
 		|| (is_o_encode_oct(o_encode))) 
 	    o_encode_stat = TRUE;
 	else if (is_o_encode_b64(o_encode)) 
 	    o_encode_stat = TRUE;
-	else if (is_o_encode_oct(o_encode)) 
-	    o_encode_stat = TRUE;
 	else if (is_o_encode_uri(o_encode)) 
 	    o_encode_stat = TRUE;
 /* encoding and inquiry */
@@ -385,6 +408,7 @@ void oconv(ch)			/* unic*de output parsing	   */
 skf_ucode ch;
 {
     skf_ucode	c1;
+    skf_ucode	c2;
     int		dd;
     int		de;
     int		x;
@@ -393,7 +417,6 @@ skf_ucode ch;
     unsigned short uukp;
 #endif
     unsigned short cnvpk;
-    skf_ucode	c2;
 
 #ifdef FOLD_SUPPORT
     int		charProp = 0;	/* character property buffer	   */
@@ -461,7 +484,8 @@ skf_ucode ch;
 		    } else if (((ch == U_KHDK) || (ch == U_LHDK)) &&
 			    (u_dakuten[c1] == 3)) {
 			ch = sgbuf + 2; sgbuf_buf = 0;
-		    } else if ((ch == U_KHDK) && (u_dakuten[c1] == 4)) {
+		    } else if (((ch == U_KHDK) || (ch == U_LHDK)) &&
+			    (u_dakuten[c1] == 4)) {
 			switch (sgbuf) {
 			    case 0x304b: ch = 0xd801; sgbuf_buf = 0; break;
 			    case 0x30ab: ch = 0xd808; sgbuf_buf = 0; break;
@@ -586,7 +610,7 @@ skf_ucode ch;
 		 ((ch >= 0xd801) && (ch <= 0xd80f)))) {
 	    cnvpk = x0201rconv(ch);
 	    if (cnvpk != 0) {
-		c2 = ((cnvpk & 0xff00U) >> 8);
+		c2 = (skf_ucode)((cnvpk & 0xff00U) >> 8);
 		ch = (skf_ucode)((cnvpk & 0x00ffU) + 0xff00UL);
 		if (c2 != 0) post_oconv(c2 + 0xff00UL);
 	    } else;
@@ -651,7 +675,7 @@ skf_ucode ch;
 		} else {
 		    c1 = A_SP; fold_count++;
 		}; 
-	      } else;
+	      } else fold_count = 0;
 #endif	/* FOLD_SUPPORT */
 	      if (!is_lineend_thru
 	      	|| (notrunc_le && is_lineend_normalize)) {
@@ -663,7 +687,7 @@ skf_ucode ch;
 		      ch = -1;		/* LF-CR: just discard		  */
 		      		/* put dummy char to prev_ch 		  */
 #endif
-#if 0
+#if 0	/* do we really need these hack? */
 		  } else if (is_nkf_compat && detect_lf 
 		  	&& (prev_ch != A_LF)) {
 		      SKFCRLF();
@@ -717,7 +741,7 @@ skf_ucode ch;
 		} else {
 		    c1 = A_SP; fold_count++;
 		};
-	      } else;
+	      } else fold_count = 0;
 #endif		/* FOLD_SUPPORT */
 	      if (!is_lineend_thru
 	      	|| (notrunc_le && is_lineend_normalize)) {
@@ -874,7 +898,7 @@ skf_ucode ch;
 	  };
 #endif
 #ifdef UNI_DECOMPOSE
-	    if (enbl_decomp) {
+	    if (enbl_decomp && (ch < 0x3400)) {
 	    	if ((uukp = nkdc_lowptr[ch - UNI_LAT_OFF]) != 0) {
 		    decompose_code(uukp);
 		    return;
@@ -1044,6 +1068,24 @@ skf_ucode ch;
 		    decompose_code(uukp);
 		    return;
 		};
+	    } else if ((ch >= 0x11090) && (ch < 0x11600)) {
+	    	/* KAITHI and other codes */
+		if ((uukp = nkdc_lowptr[ch - 0x11090 + UNI_KAITHI_OFF]) != 0) {
+		    decompose_code(uukp);
+		    return;
+		};
+	    } else if ((ch >= 0x1ee00) && (ch < 0x1ef00)) {
+	    	/* arabic mathmatical addendums */
+		if ((uukp = nkdc_lowptr[ch - 0x1ee00 + UNI_ARAM_OFF]) != 0) {
+		    decompose_code(uukp);
+		    return;
+		};
+	    } else if ((ch >= 0x1f100) && (ch < 0x1f400)) {
+	    	/* Enclosed CJK addendums and emoticons */
+		if ((uukp = nkdc_lowptr[ch - 0x1f100 + UNI_ECSP_OFF]) != 0) {
+		    decompose_code(uukp);
+		    return;
+		};
 	    } else if ((ch >= 0x2f800) && (ch < 0x2fa20)) {
 			    /* CJK cpt.sup  */
 		if ((uukp = nkdc_lowptr[ch - 0x2f800 + UNI_CJKC_OFF]) != 0) { 
@@ -1453,6 +1495,12 @@ skf_ucode c3;
 	    ((int) (d2 + ((d1 & 1) ? ((d2 < 0x60) ? 0x1f : 0x20) : 0x7e))));
 	};
 #endif
+    } else if ((is_ms_932(conv_cap) || is_ms_943(conv_cap)) &&
+    		(c3 <= 0xa878)) {
+    	d1 = ((c3 & 0x7f00U) >> 8) - 0x21 + 0x80 + 1;
+	d2 = (c3 & 0x7f);
+	SKFputc((int)(((d1 - 1) >> 1) + 0xb0));
+	SKFputc((int)(d2 + ((d1 & 1) ? ((d2 < 0x60) ? 0x1f : 0x20) : 0x7e)));
     } else 	/* should not be come here			   */
 	out_undefined(c3,SKF_OUNDEF);
 }
@@ -1594,18 +1642,26 @@ skf_ucode c3;
 	if (d2 == HZ_ANN) { SKFputc(HZ_ANN); };
 	SKFputc(d2);
     } else if (is_zW(conv_cap)) {
-	if (is_zw_shift && (d2 != A_LF) && (d2 != A_CR)) {
-	    SKFputc(A_SP);
-	} else {
-	    SKFputc(ZW_ANN); SKFputc(ZW_ENT); /* dirty but needed  */
-	    SKFputc(A_SP);
-	    set_zw_shift;
-	};
-	if (((is_lineend_crlf || is_lineend_cr || is_lineend_thru)
+	if ((d2 != A_LF) && (d2 != A_CR)) {
+	    if (is_zw_shift) {
+	    	if (d2 == A_SP) {
+		   SKFputc(ZW_LF);
+		} else SKFputc(A_SP);
+	    } else {
+		SKFputc(ZW_ANN); SKFputc(ZW_ENT); /* dirty but needed  */
+	    	if (d2 == A_SP) {
+		   SKFputc(ZW_LF);
+		} else SKFputc(A_SP);
+		set_zw_shift;
+	    };
+	} else if (
+	    ((is_lineend_crlf || is_lineend_cr || is_lineend_thru)
 		&& (d2 == A_CR)) ||
 	    ((is_lineend_lf || (is_lineend_thru && (!detect_cr)))
 		&& (d2 == A_LF))) {
-	    SKFputc(ZW_LF); res_zw_shift;
+	    if (is_zw_shift) {
+	    	SKFputc(ZW_LF); res_zw_shift;
+	    } else;
 	    if (d2 == A_CR) set_detect_cr;
 	    if (d2 == A_LF) set_detect_lf;
 	} else ;
@@ -1831,7 +1887,7 @@ const char *x;
 	else if (is_euc(conv_cap)) SKFEUCSTROUT(x);
 	else if (out_bg(conv_cap)) SKFBGSTROUT(x);
 	else if (is_ucs_ufam(conv_cap)) SKFUNISTROUT(x);
-	else if (is_ucs_brgt(conv_cap)) SKFBRGTSTROUT(x);
+	else if (is_brgt(conv_cap)) SKFBRGTSTROUT(x);
 	else if (is_keis(conv_cap)) SKFKEISSTROUT(x); 
 }
 
@@ -1846,7 +1902,7 @@ void SKF1FLSH()
 	} else if (is_euc(conv_cap)) { SKFEUC1FLSH();
 	} else if (out_bg(conv_cap)) { SKFBG1FLSH();
 	} else if (is_ucs_ufam(conv_cap)) { SKFUNI1FLSH();
-	} else if (is_ucs_brgt(conv_cap)) { SKFBRGT1FLSH();
+	} else if (is_brgt(conv_cap)) { SKFBRGT1FLSH();
 	} else if (is_keis(conv_cap)) { SKFKEIS1FLSH(); 
 	} else;
 }
@@ -1924,7 +1980,9 @@ void SKFCRLF()
 	if (is_lineend_lf) fprintf(stderr,"L");
     };
 #endif
-    if (is_lineend_thru) {
+    if (is_keis(conv_cap)) {
+    	ox_ascii_conv(A_NEL);
+    } else if (is_lineend_thru) {
 	if (use_first_detect_cr && use_detect_cr) {
 	    ox_ascii_conv(A_CR);
 	    if (use_detect_lf) ox_ascii_conv(A_LF);
@@ -1960,6 +2018,12 @@ unsigned short uu;
     int i;
     int str0;
 
+#ifdef SKFDEBUG
+    if (is_vvv_debug) {
+    	fprintf(stderr,"UU:%x ",uu);
+	(void) fflush(stderr);
+    };
+#endif
     decompose_bufp = 0;
     decompose_code_dec(uu);
     decompose_recursion_depth = 0;
@@ -1984,7 +2048,7 @@ unsigned short uu;
     int i;
     skf_ucode uukk;
     unsigned short uujj;
-    int	offset_p = -1;
+    int	offset_p = 0;
 
 #ifdef SKFDEBUG
     if (is_vvv_debug) {
@@ -1993,20 +2057,30 @@ unsigned short uu;
     };
 #endif
     uu--; 	/* Note: uu has +1 bias.	*/
+    if (uu >= nkdc_lowsize) {	/* nkdc table overflow check */
+    	out_undefined(0,SKF_ENC_ERR);
+    	return;
+    } else;
  /* limit = 32. In Unicode 5.0, longest word has 18 char decomposition */
     for (i=0;i<32 && ((uukk = nkdc_lowmap[uu]) != 0);i++, uu++) {
     	if ((uukk >= 0xa0) && (uukk < 0x3400)) {
-		offset_p = UNI_LAT_OFF;
-    	} else if ((uukk >= 0xf800) && (uukk < 0x10000)) {
-		offset_p = UNI_UP_OFF;
-    	} else if ((uukk >= 0x1d15e) && (uukk < 0x1d1c1)) {
-		offset_p = UNI_COM_OFF;
-    	} else if ((uukk >= 0x2f801) && (uukk < 0x2fa1d)) {
-		offset_p = UNI_CJKC_OFF;
-	} else ;
+		offset_p = - (UNI_LAT_OFF);
+    	} else if ((uukk >= 0xf900) && (uukk < 0x10000)) {
+		offset_p = UNI_UP_OFF - 0xf900;
+    	} else if ((uukk >= 0x1d100) && (uukk < 0x1d800)) {
+		offset_p = UNI_COM_OFF - 0x1d100;
+    	} else if ((uukk >= 0x11090) && (uukk < 0x11600)) {
+		offset_p = UNI_KAITHI_OFF - 0x11090;
+    	} else if ((uukk >= 0x1ee00) && (uukk < 0x1ef00)) {
+		offset_p = UNI_ARAM_OFF - 0x1ee00;
+    	} else if ((uukk >= 0x1f100) && (uukk < 0x1f400)) {
+		offset_p = UNI_ECSP_OFF - 0x1f100;
+    	} else if ((uukk >= 0x2f800) && (uukk < 0x2fa1d)) {
+		offset_p = UNI_CJKC_OFF - 0x2f800;
+	} else offset_p = 0;
 	
-	if ((offset_p >= 0) 
-		&& ((uujj = nkdc_lowptr[uukk-offset_p]) != 0)) {
+	if ((offset_p != 0) 
+		&& ((uujj = nkdc_lowptr[uukk+offset_p]) != 0)) {
 	    if ((decompose_recursion_depth++) > 32) {
 	    	/* Something in table is REALLY WRONG	*/
 		skferr(SKF_DECOMPERR,(long)uu,0);
@@ -2134,3 +2208,29 @@ int	oank;
 /* --------------------------------------------------------------- */
 #endif /* UNI_ENCOMPOSE */
 #endif	/* UNI_DECOMPOSE */
+
+/* --------------------------------------------------------------- */
+#ifdef SKFDEBUG
+void debugcharout(c)
+char c;
+{
+    if ((c>= 0x20) && (c <= 0x7e)) {
+    	fprintf(stderr,"(%c)",(int)c);
+    } else fprintf(stderr,"(.)");
+}
+#endif
+/* --------------------------------------------------------------- */
+void GBKR_finish_procedure()
+{
+    oconv_flush();
+    if (is_zW(conv_cap)) {
+	if (is_zw_shift) {
+	    SKFputc(ZW_LF); res_zw_shift;
+	} else;
+    } else if (is_hz(conv_cap)) {
+	if (is_hz_shift) {
+	    SKFputc(HZ_ANN); SKFputc(HZ_END);
+	};
+    } else;
+}
+/* --------------------------------------------------------------- */
