@@ -132,7 +132,8 @@ extern skf_ucode pref_subst_char;
 #define		OTBL_MONO	0x1000000UL /* Single byte type	   */
 #define		OTBL_JOHAB	0x2000000UL /* Johab		   */
 #define		OTBL_EBC	0x4000000UL /* EBCDIC Based	   */
-#define		OTBL_JIS	0x40000000UL /* JIS Kanji	   */
+#define		OTBL_JIS	0x40000000UL /* JIS Kanji & Others */
+#define		OTBL_JISK	0x20000000UL /* JIS Kanji	   */
 
 /* output table format:						   */
 /* Plane packing notation -
@@ -290,7 +291,7 @@ extern unsigned short uni_f_gb2312[];
 #define is_4pl(x)	((x & 0x8080U) == 0x0080U)
 #define is_2pl(x)	((x & 0x8080U) == 0x8000U)
 #define is_1pl(x)	((x & 0x8080U) == 0x0000U)
-#define is_2ln(x)   (((x & 0x8080U) == 0x8000U) && ((x & 0x7f00U) == 0))
+#define is_2ln(x)   (((x & 0x8000U) == 0x8000U) && ((x & 0x7f00U) == 0))
 
 #define stripchar(x)	(x & 0x7f7f)
 /* -- KEIS ------------------------------------------------------- */
@@ -724,7 +725,9 @@ extern void	show_lang_tag ();
 
 extern void	SKF_STRPUT P_((unsigned short *));
 extern void	utf8_uriout P_((skf_ucode));
-extern char	*utf8_urioutstr P_((skf_ucode));
+
+extern int	hex_conv_table[];
+
 /* ---- language tag controls ----- */
 extern void	output_language_tag_trigger P_(());
 /* ---- to make code small -------- */
