@@ -340,9 +340,18 @@ int oo_codeset;
 	} else {
 	    desc = "utf-32le"; 	
 	};
+    } else if (is_nkf_compat) {
+    	/* Note: I know THIS IS WRONG, but it's what nkf is doing  */
+    	if (oo_codeset == codeset_cp932) {
+	    desc = "shift_JIS";
+	} else if (oo_codeset == codeset_cp51932) {
+	    desc = "euc-jp";
+	} else if (oo_codeset == codeset_cp5022x) {
+	    desc = "iso-2022-jp";
+	} else;
     } else;
 
-    for (i=0; i< 16; i++, desc++) {
+    for (i=0; i< 32; i++, desc++) {
 	if (*desc == '\0') break;
 	SKFcputc(SKFtoupper(*desc));
     };
@@ -522,7 +531,7 @@ void SKFrCRLF()
 /* --------------------------------------------------------------- */
 /* hex/octal encoder						   */
 /* --------------------------------------------------------------- */
-static int hex_conv_table[16] = {
+int hex_conv_table[16] = {
   '0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F'
 };
 
@@ -720,12 +729,12 @@ int o_encode;
 /* --------------------------------------------------------------- */
 /* ace(punycode) encoder					   */
 /* --------------------------------------------------------------- */
+/*@-infloopsuncon@*/
 void o_p_encode(ch)
 skf_ucode ch;
 {
     int i;
     int	pres;
-    skf_ucode rch;
 
 #ifdef SKFPDEBUG
     if (is_vv_debug) {
@@ -735,36 +744,31 @@ skf_ucode ch;
     };
 #endif
     if (o_encode_stat == 0) {
-    	if (ch < 0) {
-	    return;
-	} else if (is_puny_strdelim(ch) || (ch == '/')) {
+	if (is_puny_strdelim(ch) || (ch == '/') || (ch < 0)) {
 		/* if basic, ch <= 0x80	 */
 		/* maybe some encoding violate pos 0x3f '/', but this */
 		/* won't be serious matter for punycode cutting. */
 	    while (!enc_pre_qempty()) SKFcputc(enc_pre_deque());
-	    SKFcputc(ch);
+	    if (ch >= 0) SKFcputc(ch);
 	    enc_pre_qreset();
-	} else if (!is_puny_basic(ch) || (ch == A_ESC)) {
+	} else if (!is_puny_basic(ch)) {
 	    enc_pre_enque(ch);
 	    o_encode_stat = TRUE;
+	    puny_maxcp = 1;
 	} else {
 	    enc_pre_enque(ch);
 	    o_encode_stat = TRUE;
 	};
     } else {
-	if ((is_puny_strdelim(ch)) || enc_pre_qfull()) {
-	    rch = ch;
-	    if (ch < 0) {
-	    	ch = 0;
-	    };
-	    enc_pre_enque(ch);
+	if ((is_puny_strdelim(ch)) || enc_pre_qfull() || (ch < 0)) {
+	/* Note: strdelim includes range 0x00 - 0x20 */
 	    encode_post_wptr = O_PSTQDEPTH;
 	    if (puny_maxcp == 0) {
 	    	while (!enc_pre_qempty()) {
 		    SKFcputc(enc_pre_deque());
 		};
 	    } else {
-		pres = punycode_encode((enc_pre_q_w_ptr - 1),
+		pres = punycode_encode((enc_pre_q_w_ptr),
 				 encode_pre_queue,
 				 &encode_post_wptr,
 				 encode_post_queue);
@@ -774,18 +778,23 @@ skf_ucode ch;
 		    SKFcputc('-'); SKFcputc('-');
 		    for (i=0;i<encode_post_wptr;i++)
 			SKFcputc(encode_post_queue[i]);
-		};
+		} else;
 	    };
 	    o_encode_stat = FALSE;
 	    enc_pre_qreset();
 	    puny_maxcp = 0;
-	    if (is_puny_delim(rch)) SKFcputc(rch);
+	    if (is_puny_strdelim(ch)) SKFrputc(ch);
+	    else if (ch >= 0) {
+	    	enc_pre_enque(ch);
+		o_encode_stat = TRUE;
+	    } else;
 	} else {
-	    if (!is_puny_basic(ch) || (ch == A_ESC)) puny_maxcp = 1;
+	    if (!is_puny_basic(ch)) puny_maxcp = 1;
 	    enc_pre_enque(ch);
 	};
     };
 }
+/*@+infloopsuncon@*/
 #endif
 
 /* --------------------------------------------------------------- */
@@ -1131,7 +1140,7 @@ int	ch;
 	    o_encode_pend = 0;
 	    return;
 	} else if (is_o_encode_oct(o_encode)) {
-	    if (!is_lineend(ch)) {
+	    if (!is_lineend(ch) && (ch >= 0)) {
 		SKFcputc(BSL_DEL);
 		char2oct(ch);
 		return;
@@ -1245,3 +1254,34 @@ skf_ucode ch;
     o_encode_lm = 0;
 }
 /* --------------------------------------------------------------- */
+void puny_finish_procedure()
+{
+    int i;
+    int	pres;
+
+    oconv_flush();
+    if ((o_encode_stat == 0) || (puny_maxcp == 0)) {
+	while (!enc_pre_qempty()) SKFcputc(enc_pre_deque());
+	o_encode_stat = FALSE;
+	enc_pre_qreset();
+	puny_maxcp = 0;
+    } else {
+    /* Note: strdelim includes range 0x00 - 0x20 */
+	encode_post_wptr = O_PSTQDEPTH;
+	pres = punycode_encode((enc_pre_q_w_ptr),
+			 encode_pre_queue,
+			 &encode_post_wptr,
+			 encode_post_queue);
+	if (pres == 0) {
+	    SKFcputc(PUNY_PRFX1);
+	    SKFcputc(PUNY_PRFX2);
+	    SKFcputc('-'); SKFcputc('-');
+	    for (i=0;i<encode_post_wptr;i++)
+		SKFcputc(encode_post_queue[i]);
+	} else;
+	o_encode_stat = FALSE;
+	enc_pre_qreset();
+	puny_maxcp = 0;
+    };
+}
+/* --------------------------------------------------------------- */
