@@ -595,6 +595,9 @@ skf_ucode c2;
 	post_oconv(0x29);
     } else if ((c1 >= 0xfc) && (c1 <= 0xfe)) { /* Enclosed kana */
 	CJK_circled(0x2ff4+c1,8);
+    } else if (c1 == 0xff) {	/* new era ligature		*/
+    	post_oconv(0x4ee4);
+    	post_oconv(0x548c);
     } else {		  /* just undefied 		   */
 	out_undefined(c2,SKF_OUNDEF);
     };
@@ -874,8 +877,8 @@ skf_ucode c1;
     	input:		c1 - input character
 	output:		none
 */
-static char *u_bra_lig_supl[10] = {
-  "DJ","CL","COOL","FREE","ID","NEW","NG","OK",
+static char *u_bra_lig_supl[11] = {
+  "DJ","CL","COOL","FREE","ID","NEW","NG","OK","SOS",
   "UP!","VS"
 };
 
@@ -1064,7 +1067,7 @@ void show_lang_tag()
 	    SKFputc((int)((ll >> 8) & 0x7f));
 	    SKFputc((int)(ll & 0x7f));
 	};
-    } else if (is_ucs_brgt(conv_cap)) { /* TAD Announcer	   */
+    } else if (is_brgt(conv_cap)) { /* TAD Announcer	   */
 	tron_announce();
     };
 }
@@ -1404,7 +1407,8 @@ skf_ucode c1;
 #endif
     if (use_latin2html) enbl = latin2html(c1);
     if (use_latin2tex && !enbl) enbl = latin2tex(c1);
-    if (use_latin2null || is_no_bfc) return;	/* discard 'em	   */
+    if ((use_latin2null && is_nkf_compat) 
+    		|| is_no_bfc) return;	/* discard 'em	   */
 
     if (!enbl) {
 	if ((c1 >= 0x3000) && (c1 <= 0x4e00) &&
@@ -1790,27 +1794,56 @@ static const char *uni_latin_html_26[16] = {  /* 0x2660 - 0x266f   */
     NULL, "&flat;", "&natur;", "&sharp;"
 };
 
+static int hex_conv_table_s[16] = {
+  '0','1','2','3','4','5','6','7','8','9','a','b','c','d','e','f'
+};
+
 int latin2html(c1)
 skf_ucode c1;
 {
     skf_ucode c2,c3;
+    skf_ucode ch;
     const char *sym = NULL;
 
 #ifdef SKFDEBUG
-	if (is_vv_debug)
-		fprintf(stderr," latin2html: %04x",c1);
+    if (is_vv_debug)
+		fprintf(stderr," latin2html: %06lx",(unsigned long)c1);
 #endif
     c2 = (c1 >> 8) & 0xff; c3 = (c1 & 0xff);
     if (((c1 < A_DEL) && (!use_htmlsanitize ||
 		(uni_latin_html_0[c1 - 0x20] == NULL))) 
 		|| (use_latin2htmld)) {
+	if (c1 >= UNI_HIGH_LIMIT) {
+	    out_undefined(c1,SKF_UNSURG);
+	    return(TRUE);
+	} else;
 	SKFSTROUT("&#");
-	printf("%03d;",c1); 	/* output 3 digit at least	   */
+	ch = c1;
+	if (c1 >= 1000000) {
+	    SKFputc(hex_conv_table_s[(c1 / 1000000)]);
+	    c1 = c1 % 1000000;
+	} else;
+	if ((c1 >= 100000) || (ch >= 1000000)) {
+	    SKFputc(hex_conv_table_s[(c1 / 100000)]);
+	    c1 = c1 % 100000;
+	} else;
+	if ((c1 >= 10000) || (ch >= 100000)) {
+	    SKFputc(hex_conv_table_s[(c1 / 10000)]);
+	    c1 = c1 % 10000;
+	} else;
+	if ((c1 >= 1000) || (ch >= 10000)) {
+	    SKFputc(hex_conv_table_s[(c1 / 1000)]);
+	    c1 = c1 % 1000;
+	} else;
+	SKFputc(hex_conv_table_s[(c1 / 100)]);
+	c1 = c1 % 100;
+	SKFputc(hex_conv_table_s[(c1 / 10)]);
+	c1 = c1 % 10;
+	SKFputc(hex_conv_table_s[c1 & 0xfU]);
+	SKFputc(';');
 	return(TRUE);
     } else if (use_latin2htmlh) {
-	SKFSTROUT("&#x");
-	printf("%03x;",c1); 	/* output 3 digit at least	   */
-	return(TRUE);
+    	;
     } else if (use_latin2htmlu) {
 	utf8_uriout(c1);
 	return(TRUE);
@@ -1827,14 +1860,6 @@ skf_ucode c1;
 	};
     } else if (c2 == 0x02) {
 	switch (c3) {
-#if 0
-	    case 0x26: sym ="&Adot;"; break;
-	    case 0x27: sym ="&adot;"; break;
-	    case 0x2e: sym ="&Odot;"; break;
-	    case 0x2f: sym ="&odot;"; break;
-	    case 0x32: sym ="&Ymacr;"; break;
-	    case 0x33: sym ="&ymacr;"; break;
-#endif
 	    case 0xbc: sym ="&apos;"; break;
 	    case 0xc6: sym ="&circ;"; break;
 	    case 0xc7: sym ="&caron;"; break;
@@ -1894,9 +1919,20 @@ skf_ucode c1;
 
     if (sym != NULL) {
 	SKFSTROUT(sym);
-    } else {
-	SKFSTROUT("&#");
-	printf("%03d;",c1); 	/* output 3 digit at least	   */
+    } else {	/* not dec or hex. go hex anyway		  */
+	SKFSTROUT("&#x");
+	if (c1 >= 0x10000) {
+	    SKFputc(hex_conv_table_s[(c1 >> 20) & 0x0fU]);
+	    SKFputc(hex_conv_table_s[(c1 >> 16) & 0x0fU]);
+	    c1 &= 0xffffU;
+	};
+	if (c1 >= 0x100) {
+	    SKFputc(hex_conv_table_s[(c1 >> 12) & 0x0fU]);
+	};
+	SKFputc(hex_conv_table_s[(c1 >> 8) & 0x0fU]);
+	SKFputc(hex_conv_table_s[(c1 >> 4) & 0x0fU]);
+	SKFputc(hex_conv_table_s[(c1 & 0x0fU)]);
+	SKFputc(';');
     };
 
     return(TRUE);
@@ -2188,7 +2224,7 @@ skf_ucode c1;
 	if (is_vv_debug)
 		fprintf(stderr," latin2tex: %04x",c1);
 #endif
-    c2 = (c1 >> 8) & 0xff; c3 = (c1 & 0xff);
+    c2 = (c1 >> 8) & 0xffff; c3 = (c1 & 0xff);
     if (c2 == 0) {
 	if ((c3 == 0x5c) && !use_htmlsanitize) {
 	    sym = "\\\\";
