@@ -54,7 +54,7 @@
 #define	is_det_utf7 is_ucs_utf7(i_codeset[in_codeset].encode)
 #define	is_det_keis is_keis(i_codeset[in_codeset].encode)
 #define	is_det_latin is_jis(i_codeset[in_codeset].encode)
-#define	is_det_brgt is_ucs_brgt(i_codeset[in_codeset].encode)
+#define	is_det_brgt is_brgt(i_codeset[in_codeset].encode)
 #define	is_det_transp is_ucs_transp(i_codeset[in_codeset].encode)
 
 #define is_code_undefine (in_codeset < 0) 
@@ -708,6 +708,7 @@ int preConvert (f)
     if (kuni_opt) {	/* limit detection to JIS, EUC and SJIS	   */
 	utf8fault = TRUE; utf7fault = TRUE; keisfault = TRUE;
 	latinfault = TRUE; det_limit = 2; lk_power = 1;
+	b5fault = TRUE;
     } else if (fuzzy_detect) {
 	lk_power = 2;
 	det_limit = DET_LIMIT * lk_power;
@@ -1435,15 +1436,17 @@ sensed if it fits in ISO-2022 codeset-call sequence.
 	/* check intermediate seqs. */
 /* --- UTF-8 ------------------ */
 	if (is_lang_unicode && (utf8fault == 0)) {
-	    if ((c2 == 0xbb) && (rc1 == 0xef)) { /* utf8	   */
+	    if (Qle2 && (c3 == 0xef) && (c2 == 0xbb) && (rc1 == 0xbf)) {
 		utf8likely += 3;	/* UTF8 BOM		   */
 		utf8prech = sEOF; utf8curch = sEOF;
 		has_bom = TRUE;
 	    } else if ((c2 == A_ESC) && (rc1 > 0x20) && (rc1 < 0x30)) {
 		utf8fault++;
 	    } else if ((utf8curch = is_valid_utf8_seq(rc1,c2,c3,c4)) >= 0) {
-		if ((utf8curch == 0) || (rc1 < A_DEL)) {
+		if (utf8curch == 0) {
 		    utf8ccnt = 1;
+		} else if (rc1 < A_DEL) {
+		    utf8ccnt = 0;	/* get one utf8 char	   */
 		} else {
 		    utf8ccnt = 0;	/* get one utf8 char	   */
 		    utf8likely++;
@@ -1883,7 +1886,7 @@ sensed if it fits in ISO-2022 codeset-call sequence.
 	      };
 	   };
 	} else {
-	  hzunlikely++; b5unlikely++; 
+	  hzunlikely++; 
 	};
 
 /* --- iso-8859-1, also handle ascii --- */
@@ -2228,7 +2231,7 @@ sensed if it fits in ISO-2022 codeset-call sequence.
 		printf("%s: ",in_file_name);
 	};
 #endif
-	dump_name_of_code(was_ascii,0, has_bom);
+	dump_name_of_code(was_ascii,1, has_bom);
 #ifndef SWIG_EXT
 	if (input_hard_inquiry) {
 	    dump_name_of_lineend(le_detect,0);
