@@ -1,5 +1,5 @@
-#ident "$Id: skf.c,v 1.267 2016/02/07 14:28:02 seiji Exp seiji $"
-const char *rev = "SKF version 2.00.4 2016-02-05\n";
+#ident "$Id: skf.c,v 1.268 2016/06/11 13:43:55 seiji Exp seiji $"
+const char *rev = "SKF version 2.00.17 2019-12-29\n";
 /* *******************************************************************
 ** Copyright (c) 1993-2015 Seiji Kaneko. All rights reserved.
 ** Everyone is permitted to use this program in source and binary
@@ -192,7 +192,7 @@ unsigned long	skf_given_lang = 0;	/* parametered language	 */
 unsigned long	skf_in_text_type = 0;
 
 /* control variables */
-skf_ucode	ucode_undef = 0;	/* substitute character	 */
+skf_ucode	ucode_undef = -1;	/* substitute character	 */
 int		mime_fold_llimit = MIME_ENCODE_LLIMIT;
 				/* MIME line clip length	 */
 
@@ -216,7 +216,8 @@ static int	out_code = -1;		/* internal parse pass	  */
 
 #ifndef SWIG_EXT
 char		*in_file_name;	/* input file name save area 	  */
-char		*out_file_name;	/* output file name save area 	  */
+char		*out_file_name = NULL;
+				/* output file name save area 	  */
 char		*inb_file_name;	/* input file name save area 	  */
 char		*outb_file_name = NULL; 
 				/* output file name save area 	  */
@@ -397,8 +398,13 @@ static const struct long_option option_table[] = {
     {"force-private-idn-out",128},
     {"suppress-space-convert",131},
     {"disable-space-convert",131},
+    {"suppress-space-conv",131},
+    {"disable-space-conv",131},
     {"enable-space-single-convert",132},
     {"enable-space-convert",133},
+    {"enable-ascii-convert",134},
+    {"enable-space-single-conv",132},
+    {"enable-space-conv",133},
     {"enable-ascii-conv",134},
     {"lineend-crlf",141},
     {"lineend-cr",140},
@@ -433,9 +439,9 @@ static const struct long_option option_table[] = {
     {"unicode-kana-concat",173},
     {"kana-concat",173},
     {"disable-kana-concat",174},
+    {"input-crlf",177},
     {"input-cr",175},
     {"input-lf",176},
-    {"input-crlf",177},
     {"nkf-compat",180},
     {"skf-compat",181},
     {"mime-nkf-mode",184},
@@ -446,10 +452,10 @@ static const struct long_option option_table[] = {
     {"x0212",2},
     {"fj",185},
     {"kana-convert",189},
-    {"hiragana",197},
-    {"katakana",198},
     {"hiragana-katakana",199},
     {"katakana-hiragana",199},
+    {"hiragana",197},
+    {"katakana",198},
     {"exec-in",192},
     {"exec-out",192},
     {"cp932",194},
@@ -472,8 +478,9 @@ static const struct long_option option_table[] = {
     {"mime-input",DEC_INDEX+8},
     {"mime",DEC_INDEX + 6},
     {"fb-html",120},
-    {"fb-skip",133},
-    {"fb-subchar",0x1c0},
+    {"fb-xml",124},
+    {"fb-skip",135},
+    {"fb-subchar",0x152},
     {"fb-",192},
     {"start-kanji",190},
     {"invis-strip",191},
@@ -511,6 +518,7 @@ static const struct long_option option_table[] = {
     {"itext-ftext",275},
     {"itext-man",276},
     {"itext-sgml",276},
+    {"itext-tex",277},
     {"oldcell-to-emoticon",280},
 #if defined(SWIG_EXT) && defined(SKF_PYTHON3)
     {"py-out-binary",282},
@@ -963,6 +971,8 @@ const struct long_option codeset_option_code[] = {
     {"pck",		codeset_sjis},
     {"us",		codeset_ascii},	
     {"646",		codeset_ascii},	
+    {"932",		codeset_cp932}, 
+    {"943",		codeset_cp943}, 
     {"jp",		codeset_x0208},
     {"guess",		-1},
     {"?",99},
@@ -992,9 +1002,9 @@ static const struct long_option encode_option_str[] = {
     {"hex-perc-encode",3},
     {"hex-decode",1},
     {"hex-perc-decode",3},
+    {"uri-docomo",65},
     {"uri",64},
     {"url",64},
-    {"uri-docomo",65},
     {"hex",0x1},
     {"cap",0x1},
 #if defined(ROT_SUPPORT) && !defined(SWIG_EXT)
@@ -1039,15 +1049,20 @@ int main(argc, argv)
     mode_t	oldmode = 0;
     int		ofd;
     int		locale_codeset;
+#ifdef DYNAMIC_LOADING
+    int		actual_out_code;
+#endif
 #ifdef SKFDEBUG
     int sy;
 #endif
 #if	defined(NKF_NAMETEST) && !defined(SWIG_EXT)
     char *name_me = argv[0];
-    int i;
+    int i,j;
+#if defined(_WIN32) || defined(__MINGW32__)
+    int k;
 #endif
-#ifndef SWIG_EXT
 #endif
+    int		nounlink_inbfile = FALSE;
 
     res_out_code;
     uni_table_init();
@@ -1056,7 +1071,7 @@ int main(argc, argv)
     fin = skf_file_buf;		/* allocate fin pointer area	   */
     in_file_name = (char *) calloc(IN_FILE_NAME,sizeof(char));
     inb_file_name = (char *) calloc(IN_FILE_NAME,sizeof(char));
-    outs_file_name = (char *) calloc(OUT_FILE_NAME,sizeof(char));
+    outs_file_name = (char *) calloc(OUT_FILE_NAME+2,sizeof(char));
 
     if ((in_file_name == NULL) || (inb_file_name == NULL)
     	|| (outs_file_name == NULL)) {
@@ -1066,7 +1081,26 @@ int main(argc, argv)
 
 #if	defined(NKF_NAMETEST) && !defined(SWIG_EXT)
     {
-	for (i = 0;i<SKF_TABLE_PATH && (name_me[i] != 0);i++) {
+    	j = skf_strlen(name_me,SKF_TABLE_PATH);
+#if defined(_WIN32) || defined(__MINGW32__)
+	k = skf_strlen(SYSTEM_EXEEXT,6);
+	for (i = 0;i<SKF_TABLE_PATH && (name_me[i] != 0)
+			&& (i <= j);i++) {
+	    if ((i >= 2) && (k == 0) && (name_me[i-2] == 'n') 
+		&& (name_me[i-1] == 'k') && (name_me[i] == 'f')
+		&& (name_me[i+1] == 0)) {
+		set_nkf_compat; is_name_nkf = TRUE;
+		break;
+	     } else if ((i >= 3+k) && (name_me[i-(2+k)] == 'n') 
+		&& (name_me[i-k-1] == 'k') && (name_me[i-k] == 'f')
+		&& (name_me[i-k+1] == '.') && (name_me[i+1] == 0)) {
+		set_nkf_compat; is_name_nkf = TRUE;
+		break;
+	    };
+	};
+#else
+	for (i = 0;i<SKF_TABLE_PATH && (name_me[i] != 0)
+			&& (i <= j);i++) {
 	    if ((i >= 2) && (name_me[i-2] == 'n') 
 		&& (name_me[i-1] == 'k') && (name_me[i] == 'f')
 		&& (name_me[i+1] == 0)) {
@@ -1074,6 +1108,7 @@ int main(argc, argv)
 		break;
 	    };
 	};
+#endif
     };
 
     if (is_nkf_compat) {
@@ -1085,6 +1120,7 @@ int main(argc, argv)
 	set_lineend_normalize;
 	set_pux212;
 	set_input_jp_limit;
+	reset_hk_enbl;
     } else;
 #endif
 #ifdef HAVE_GETENV		/* environment argument eval.      */
@@ -1117,7 +1153,7 @@ int main(argc, argv)
 #ifndef SWIG_EXT
     	locale_codeset = get_output_locale();
 	if (locale_codeset < 0) {
-	    out_undefined(0,SKF_NOOUT);
+	    out_undefined(sOCD,SKF_NOOUT);
 	    out_code = -1;
 	} else out_code = locale_codeset;
 #else
@@ -1125,6 +1161,18 @@ int main(argc, argv)
 #endif
     } else;
 
+#ifdef DYNAMIC_LOADING
+    /* Note: B-Right/V is not ISO-2022 type codeset, but it needs  */
+    /*   iso-2022-kr code for hangul processing. This treatment	   */
+    /*   would be a sheer hack, but proper handling needs a table  */
+    /*   duplication for just to straight go without exceptions.   */
+    if (out_code == codeset_brgt) {
+    	actual_out_code = out_code;
+	out_code = codeset_euckr;
+	skf_charset_parser(out_code); 
+	out_code = actual_out_code;
+    } else;
+#endif
     if (out_code < 0) out_code = DEFAULT_O;
 
     skf_charset_parser(out_code); 
@@ -1212,15 +1260,23 @@ int main(argc, argv)
 /* --- input/output initialization --- */
 	skf_ioinit(fout);
     } else {	/* nkf_overwrite or nkf_in_place		   */
+	if (outs_file_name[0] != '\0') {
+	    nounlink_inbfile = TRUE;
+	} else;
+        if (out_file_name == NULL) {
 #if defined(__MINGW32__)
-	out_file_name = (char *) calloc(IN_FILE_NAME+MAX_PATH+20,
+	    out_file_name = (char *) calloc(IN_FILE_NAME+MAX_PATH+20,
 					sizeof(char));
 	/* 20=filename("skfuuuu.tmp"=11) + Margin(8) + "\0"(1) */
 #else
-	out_file_name = (char *) calloc(IN_FILE_NAME+SUFFIX_LIMIT,
+	    out_file_name = (char *) calloc(IN_FILE_NAME+SUFFIX_LIMIT,
 					sizeof(char));
 #endif
-	outb_file_name = (char *) calloc(OUT_FILE_NAME,sizeof(char));
+	} else;
+	if (outb_file_name == NULL) {
+	    outb_file_name = (char *) calloc(OUT_FILE_NAME,
+	    	sizeof(char));
+	} else;
 	if ((out_file_name == NULL) || (outs_file_name == NULL)
 		|| (outb_file_name == NULL)) {
 	    skferr(SKF_MALLOCERR,(long)0,(long)0);
@@ -1245,6 +1301,7 @@ int main(argc, argv)
 	    if ((fin = skf_fopen(in_file_name,SKF_INMODE)) == NULL) {
 		skf_openerr(in_file_name,0);
 		skfres = -1;
+		c++; continue;
 	    } else {
 		if (is_nkf_overwrite || is_nkf_in_place) {
 		    strncpy(inb_file_name,in_file_name,
@@ -1266,7 +1323,7 @@ int main(argc, argv)
 			skf_exit(EXIT_FAILURE);
 		    } else;
 			/*@+voidabstract@*/
-		    if ((fout = (skfoFILE *)fdopen(ofd,skf_outmode))
+		    if ((fout = (skfoFILE *)skf_fdopen(ofd,skf_outmode))
 		    		== NULL) {
 			skf_openerr(out_file_name,1);
 			skf_exit(EXIT_FAILURE);
@@ -1307,9 +1364,14 @@ int main(argc, argv)
 			} else;
 		    } else;
 
-		    if (unlink(inb_file_name) < 0) {
-		    	skf_openerr(inb_file_name,5);
-			skf_exit(EXIT_FAILURE);
+		    if (nounlink_inbfile == FALSE) {
+			if (unlink(inb_file_name) < 0) {
+			    skf_openerr(inb_file_name,5);
+			    if (unlink(out_file_name) < 0) {
+				skf_openerr(out_file_name,6);
+			    } else;
+			    skf_exit(EXIT_FAILURE);
+			} else;
 		    } else;
 
 		    if (rename(out_file_name,outb_file_name) < 0) {
@@ -1319,9 +1381,6 @@ int main(argc, argv)
 		    } else;
 		    (void)umask(oldmode);
 		} else;
-		if (outb_file_name != NULL) {
-		    free(outb_file_name);
-		} else;
 	    };
 	    c++;
 	};
@@ -1363,6 +1422,7 @@ int main(argc, argv)
 	if (out_file_name != NULL) free(out_file_name); /* ??? */
     };
     if (outs_file_name != NULL) free(outs_file_name);
+    if (outb_file_name != NULL) free(outb_file_name);
 #endif
 /*@+branchstate@*/ /*@+usedef@*/
 
@@ -1379,7 +1439,8 @@ skfFILE *fin;
 {
     int	us = sEOF;
 
-    skf_input_lang = skf_get_langcode(skf_output_lang);
+    if (skf_given_lang != 0) skf_input_lang = skf_given_lang;
+    else skf_input_lang = skf_get_langcode(skf_output_lang);
 
     while (TRUE) {
 	if ((us = preConvert(fin)) == sEOF) break;
@@ -1446,14 +1507,14 @@ int argeval(pargc,pargv,limit)
 		unbuf_f = FALSE; continue;
 	    case 'u':   /* unbuffered in			   */
 		unbuf_f = TRUE; continue;
-	    case 'n':   /* 7-bit JIS X 0208(1983)                  */
-	    	if (is_nkf_compat) {
-		    error_extend_option(SKF_NKFINCOMPAT,cp-1);
-		    fail = TRUE;
-		} else out_code = codeset_x0208;
-		continue;
+	    case 'n':	/*FALLTHROUGH*/
 	    case 'j': 
-		out_code = codeset_x0208; continue;
+		if (is_nkf_compat) {
+		    out_code = codeset_cp5022x;
+		} else {
+		    out_code = codeset_x0208;
+		};
+		continue;
 	    case 'a':   /* 8-bit EUC-code with JIS X 0208(1983)    */
 	    	if (is_nkf_compat) {
 		    error_extend_option(SKF_NKFINCOMPAT,cp-1);
@@ -1527,6 +1588,7 @@ int argeval(pargc,pargv,limit)
 		    continue;
 		} else {
 		    error_extend_option(SKF_NKFINCOMPAT,cp-1);
+		    cp++;
 		    fail = TRUE;
 		};
 		break;
@@ -1611,7 +1673,7 @@ int argeval(pargc,pargv,limit)
 			set_base64_encode;
 			cp++;
 		    } else if (*cp == 'Q') {
-			set_mimeq_encode;
+			set_hex_qencode;
 			cp++;
 		    } else if (*cp == 'S') {
 			set_mimeb_strict;
@@ -1630,6 +1692,9 @@ int argeval(pargc,pargv,limit)
 			    set_use_iso8859;
 			    in_codeset = codeset_8859_1;
 			    out_code = codeset_8859_1;
+			    parse = SG1_INDEX + SK_UB_8859+iso8859_1_index;
+			    skf_codeset_parser(parse);
+			    set_euc_protect;
 			} else {
 			    error_extend_option(SKF_NKFINCOMPAT,cp-1);
 			    fail = TRUE;
@@ -1734,9 +1799,11 @@ int argeval(pargc,pargv,limit)
 		if (*cp == 'B') {
 		    set_o_base64_encode;
 		    cp++;
+		} else if (*cp == 'Q') {
+		    set_o_hex_qencode;
+		    cp++;
 		} else {
 		    set_o_mimeb_encode;
-		    if (*cp == 'Q') cp++;
 		};
 		continue;
 	    case 'L':
@@ -1748,11 +1815,18 @@ int argeval(pargc,pargv,limit)
 		    set_lineend_crlf; cp++;
 		} else {
 		    set_lineend_thru;
+		    if (*cp == '0') cp++;
 		};
 		continue;
 	    case 'g':
 	    	if (is_nkf_compat) {
 		    set_input_inquiry; set_estab_reinit;
+		    if ((*cp >= '0') && (*cp <= '9')) {
+		    	if (*cp >= '2') {
+			    set_input_hard_inquiry;
+			} else;
+			cp++;
+		    } else;
 		} else {
 		    error_extend_option(SKF_NKFINCOMPAT,cp-1);
 		    fail = TRUE;
@@ -1805,7 +1879,7 @@ int argeval(pargc,pargv,limit)
 		    	set_nkf_jffbroken; cp++;
 		    } else {
 		    	set_nkf_jbroken;
-			if (*cp == '0') cp++;
+			if ((*cp >= '0') && (*cp <= '9')) cp++;
 		    };
 		    in_codeset = codeset_x0208;
 		    continue;
@@ -1814,6 +1888,12 @@ int argeval(pargc,pargv,limit)
 		    fail = TRUE;
 		};
 		continue;
+	    case 't':
+	    	if ((*cp >= '0') && (*cp <= '1')) {
+		    out_code = codeset_binary;
+		    cp++;
+		} else;
+		continue;
 	    case 'V':	/*FALLTHROUGH*/
 	    case 'v':   /* display revision information            */
 	        if (*cp == 'v') {	/* -vv			   */
@@ -1822,7 +1902,7 @@ int argeval(pargc,pargv,limit)
 		skf_terminate(EXIT_SUCCESS);
 #ifdef SKFDEBUG
 	    case '%':   /* for debug only                          */
-		if ((*cp != '\0') && (*cp <= '9')) {
+		if ((*cp != '0') && (*cp <= '9')) {
 		  if (*cp > '2') { set_vvv_debug;
 		  } else if (*cp > '1') { set_vv_debug;
 		  } else set_v_debug;
@@ -1864,7 +1944,7 @@ int argeval(pargc,pargv,limit)
 		  case 19: set_keis_detect_untest; break;
 		  case 21: set_miti_undef; break; 
 		  case 22: set_old_nec_compat; break;
-		  case 25: set_use_uni_repl; break;
+		  case 25: ucode_undef = UCODE_REPLACE; break;
 		  case 26: set_quad_char; break;
 		  case 27: set_sup_jis90; break;
 		  case 28: set_chart_dsbl; break;
@@ -1911,7 +1991,7 @@ int argeval(pargc,pargv,limit)
 		      conv_cap = 0; conv_alt_cap = 0;
 		      skf_input_lang = 0;
 		      skf_output_lang = 0; skf_given_lang = 0;
-		      ucod_flavor = 0; ucode_undef = 0;
+		      ucod_flavor = 0; ucode_undef = -1;
 		      res_debug; codeset_flavor = 0;
 		      option_guarding = 0; unbuf_f = 0;
 		      in_codeset = -1; 
@@ -2008,6 +2088,7 @@ int argeval(pargc,pargv,limit)
 			set_lineend_normalize;
 			set_pux212;
 			set_input_jp_limit;
+			reset_hk_enbl;
 			break;
 		  case 181: reset_nkf_compat;
 			res_sup_space_conv;
@@ -2085,6 +2166,7 @@ int argeval(pargc,pargv,limit)
 		  case 210: set_nkf_in_place; 	/*@FALLTHROUGH@*/
 		  case 211: skf_fileout = TRUE;
 		  	set_nkf_overwrite;
+			set_estab_reinit ;
 			cq = strchr(cp,'=');
 			if (cq != NULL) {
 			    cq++;
@@ -2129,7 +2211,8 @@ int argeval(pargc,pargv,limit)
 		      skf_terminate(EXIT_FAILURE);
 		};
 	    } else {
-	      if ((parse <= M_PLIMIT) || (parse == ENC_INDEX)
+	      if (((parse > M_PLIMIT) && (parse >= CST_INDEX))
+	        || (parse == ENC_INDEX)
 	        || (parse == CST_INDEX) || (parse == DST_INDEX)
 		|| (parse == LNG_INDEX) || (parse == DEC_INDEX)
 		|| (parse == SG0_INDEX) || (parse == SG1_INDEX)
@@ -2137,11 +2220,12 @@ int argeval(pargc,pargv,limit)
 		cq = strchr(cp,'=');
 		if ((cq == NULL) || (*(++cq) == '\0')) {
 		    if (pargc > 1) { /* has at least one more  */
-			cq = *(++pargv); --pargc;
-			if (cq == NULL) {
-			    error_extend_option(SKF_NOCSET,"NULL");
-			    skf_terminate(EXIT_FAILURE);
+			cq = *(pargv + 1);
+			if ((cq == NULL) || (*cq == '-')) {
+			    ++pargv; --pargc;
+			    continue;
 			};
+			++pargv; --pargc;
 		    } else {
 			error_extend_option(SKF_NOCSET,"NULL");
 			skf_terminate(EXIT_FAILURE);
@@ -2194,7 +2278,7 @@ int argeval(pargc,pargv,limit)
 			skf_terminate(EXIT_FAILURE);
 		    };
 		};
-		if (parse > M_PLIMIT) parse += pcode;
+		if ((parse > M_PLIMIT) && (parse != LNG_INDEX)) parse += pcode;
 	      };
 	      if ((parse >= CST_INDEX) && (parse <= DST_ILIMIT)) {
 		  if ((parse >= CST_INDEX) && (parse <= CST_ILIMIT)) {
@@ -2254,9 +2338,7 @@ int argeval(pargc,pargv,limit)
 		};
 	      } else if ((parse >= LNG_INDEX) && (parse <= LNG_ILIMIT)) {
 			/* setting language parser		   */
-		parse = parse & 0x5f5f;	
-			/* capitalize by resetting bit5 and 7	   */
-		skf_given_lang = parse;
+		skf_given_lang = (unsigned long)pcode;
 	      } else if ((parse >= SG0_INDEX) && (parse <= SG_ILIMIT)) {
 		  skf_codeset_parser((int)(parse & SG_PH_MASK));
 	      } else if (parse <= M_PLIMIT) {
@@ -2274,7 +2356,8 @@ int argeval(pargc,pargv,limit)
 	      };
 	    };
 	} else {	/* -- only				   */
-		return(--pargc);	/* indicate option end	   */
+	    pargv++; pargc--;
+	    return(++pargc);	/* indicate option end	   */
 	};
       };
       pargv++; pargc--;
@@ -2330,7 +2413,11 @@ skfoFILE *fout;
     else if (is_ucs_utf8(conv_cap)) utf8_finish_procedure();
     else if (is_ucs_utf16(conv_cap)) ucod_finish_procedure();
     else if (out_bg(conv_cap)) BG_finish_procedure();
-    else if (is_ucs_brgt(conv_cap)) BRGT_finish_procedure();
+    else if (is_keis(conv_cap)) KEIS_finish_procedure();
+    else if (is_big5fam(conv_cap)) BG_finish_procedure();
+    else if (is_brgt(conv_cap)) BRGT_finish_procedure();
+    else if (is_gbkrfam(conv_cap)) GBKR_finish_procedure();
+    else if (is_ucs_puny(conv_cap)) puny_finish_procedure();
     else;
 
 /* if any characters hang in encoder, push out these stuffs. 	   */
@@ -2351,6 +2438,7 @@ skfoFILE *fout;
     SKFfflush((skfoFILE *)fout);
     if (o_encode) encoder_tail();
     SKFfflush((skfoFILE *)fout);
+    init_all_stats();
 }
 #endif
 
@@ -2431,7 +2519,7 @@ void skf_script_init()
     debug_opt = 0;
 
     /* control variables */
-    ucode_undef = 0;	/* undefine in unic*de	 */
+    ucode_undef = -1;	/* undefine in unic*de	 */
 
     init_all_stats();
 
