@@ -68,6 +68,7 @@
 #define         A_CR	0x0d
 #define         A_SO	0x0e
 #define         A_SI	0x0f
+#define         A_NEL	0x15
 #define         A_CAN	0x18
 #define         A_SUB	0x1a
 #define         A_ESC	0x1b
@@ -79,6 +80,7 @@
 #define         A_0	0x30
 #define         A_BRA	0x3c
 #define         A_KET	0x3e
+#define         A_QUE	0x3f
 #define         A_AT	0x40
 #define         A_CAPA	0x41
 #define         A_CAPZ	0x5a
@@ -110,6 +112,7 @@
 #define		ASCII_END 0x80
 #define		A_KDEL	0xff
 #define		MS_GAIJI 0xf0
+#define		UCODE_REPLACE 0xfffd
 
 /* X-0208 assignment						   */
 #define		A_MAX	0x28	/* maximum alpha code		   */
@@ -225,6 +228,7 @@
 #define		LBMSK	0x0ffU
 #define		HBMSK	0xff00U
 
+#define FIXSTRLEN_LIMIT	64
 /* --------------------------------------------------------------  */
 /* internal types						   */
 /* --------------------------------------------------------------  */
@@ -419,6 +423,7 @@ extern short	debug_opt;
 #define SKF_GBKR	0x00a0UL    /* GBKR-derivative style codes */
 #define SKF_NNISO	0x00c0UL    /* Various misc code sets	   */
 #define SKF_KEIS 	0x00e0UL    /* output is kind of KEIS	   */
+#define SKF_CDSP 	0x00f0UL    /* output is not ASCII/EBCDIC  */
 
 /* --------------------------------------------------------------- */
 /* misc selectors 0x0000000*					   */
@@ -497,14 +502,12 @@ extern short	debug_opt;
 #define SKF_UCS_UTF7	0x0006UL    /* BASE utf-7		   */
 #define SKF_UCS_PUNY	0x0008UL    /* BASE IDN Punycode	   */
 #define SKF_UCS_URI	0x0009UL    /* BASE IDN URI notation	   */
-#define SKF_UCS_BRGT	0x000eUL    /* output is B/Right V	   */	
 #define SKF_UCS_TRANS	0x000fUL    /* transparent mode		   */
 
 #define is_ucs_utf16(x) ((x & SKF_COD_UC_MSK)==(SKF_UCS|SKF_UCS_UTF16))
 #define is_ucs_utf32(x) ((x & SKF_COD_O_MSK)==(SKF_UCS|SKF_UCS_UTF32))
 #define is_ucs_utf8(x) ((x & SKF_COD_O_MSK)==(SKF_UCS|SKF_UCS_UTF8))
 #define is_ucs_utf7(x) ((x & SKF_COD_O_MSK)==(SKF_UCS|SKF_UCS_UTF7))
-#define is_ucs_brgt(x) ((x & SKF_COD_O_MSK)==(SKF_UCS|SKF_UCS_BRGT))
 #define is_ucs_puny(x) ((x & SKF_COD_O_MSK)==(SKF_UCS|SKF_UCS_PUNY))
 #define is_ucs_uri(x) ((x & SKF_COD_O_MSK)==(SKF_UCS|SKF_UCS_URI))
 #define is_ucs_transp(x) ((x & SKF_COD_O_MSK)==(SKF_UCS|SKF_UCS_TRANS))
@@ -637,6 +640,15 @@ extern short	debug_opt;
 #define is_keis_melc(x) ((x & SKF_COD_O_MSK) == (SKF_KEIS | SKF_MELC))
 #define is_keis_keis(x) ((x & SKF_COD_O_MSK) == SKF_KEIS)
 
+/* --------------------------------------------------------------- */
+/* 10th set: SKF_CDSP						   */
+/* --------------------------------------------------------------- */
+#define out_cdsp  ((conv_cap & SKF_COD_C_MSK) == SKF_CDSP)
+
+#define SKF_CDSP_BRGT	0x00000001UL    /* output is B/Right V	  */	
+
+#define is_brgt(x)	((x & SKF_COD_O_MSK) == (SKF_CDSP | SKF_CDSP_BRGT))
+
 #define is_cod_iso(x) ((((x & SKF_COD_CAT_MSK) == 0) || is_msfam(x) \
 			|| is_hz(x) || is_zW(x)) \
 			&& !is_euc_niso(x))
@@ -722,7 +734,7 @@ extern int	skip_x221_4_null();
 #define	USE_JIS8 0x00001000UL	/* iso-2022 8bit enable		   */
 
 /* --------------------------------------------------------------- */
-/* should be not needed.					   */
+/* should not be needed.					   */
 /* --------------------------------------------------------------- */
 #define COD_S_90  0x00000400UL	/* code is not JIS X0208(1990)	   */
 #define sup_jis90 (conv_alt_cap & COD_S_90)
@@ -766,7 +778,7 @@ extern int	skip_x221_4_null();
 #define set_fold_flat conv_alt_cap |= FOLD_FLAT
 #define reset_fold_flat conv_alt_cap &= 0xfffffffbUL
 
-#define FOLD_STRONG 0x00000001UL /* strong kinsoku		   */
+#define FOLD_STRONG 0x00000002UL /* strong kinsoku		   */
 #define fold_strong (conv_alt_cap & FOLD_STRONG)
 #define set_fold_strong conv_alt_cap |= FOLD_STRONG
 #define reset_fold_strong conv_alt_cap &= 0xfffffffdUL
@@ -1007,7 +1019,7 @@ extern int 	o_encode;
 #define	O_ENCODE_URI	0x0200U
 #define	O_ENCODE_DCM	0x0400U	/* not used for output		   */
 #define	O_ENCODE_Q	0x0800U /* hex but uses "=" 		   */
-#define	O_ENCODE_PUNY	0x1000U /* hex but uses "=" 		   */
+#define	O_ENCODE_PUNY	0x1000U /* PUNYCODE			   */
 
 #define O_ENCODE_MIMES  (O_ENCODE_MIME | O_ENCODE_MIMEQ | O_ENCODE_RFC2231)
 #define O_ENCODE_MIMEBQ  (O_ENCODE_MIME | O_ENCODE_MIMEQ)
@@ -1201,6 +1213,10 @@ extern unsigned long	ucod_flavor;
 #define set_uni_kana_concat ucod_flavor |= KANA_CONCAT
 #define res_uni_kana_concat ucod_flavor &= 0xfffff7ffUL
 
+#define OLDCELL_TO_EMOT 0x00001000UL
+#define enable_cellconvert (ucod_flavor & OLDCELL_TO_EMOT)
+#define set_enable_cellconvert ucod_flavor |= OLDCELL_TO_EMOT
+
 #define DSBL_UCOD_ENCOMP 0x00002000UL	/* Unicode KANAcompress	*/
 #define dsbl_ucod_encomp (ucod_flavor & DSBL_UCOD_ENCOMP)
 #define set_dsbl_ucod_encomp ucod_flavor |= DSBL_UCOD_ENCOMP
@@ -1220,10 +1236,6 @@ extern unsigned long	ucod_flavor;
 #define enable_dbl_latin (ucod_flavor & ENBL_DBL_LATIN)
 #define set_enable_dbl_latin ucod_flavor |= ENBL_DBL_LATIN
 
-#define OLDCELL_TO_EMOT 0x00020000UL
-#define enable_cellconvert (ucod_flavor & OLDCELL_TO_EMOT)
-#define set_enable_cellconvert ucod_flavor |= OLDCELL_TO_EMOT
-
 #define ENBL_VAR_CNTL 0x00040000UL
 #define enable_var_cntl (ucod_flavor & ENBL_VAR_CNTL)
 #define set_enable_var_cntl ucod_flavor |= ENBL_VAR_CNTL
@@ -1459,8 +1471,8 @@ extern unsigned long	nkf_compat;
 #define set_nkf_c_katakana nkf_compat |= NKF_KATAKANA
 
 #define is_nkf_no_hk	(nkf_compat & NKF_FORCE_NOHK)
-#define set_nkf_no_hk	nkf_compat = (nkf_compat & 0xffefffffUL) | NKF_FORCE_NOHK
-#define reset_nkf_no_hk	nkf_compat = (nkf_compat & 0xffefffffUL) 
+#define set_nkf_no_hk	nkf_compat = (nkf_compat & 0xfdffffffUL) | NKF_FORCE_NOHK
+#define reset_nkf_no_hk	nkf_compat = (nkf_compat & 0xfdffffffUL) 
 #define is_o_prefer_ascii	(nkf_compat & O_PREFER_ASC)
 #define set_o_prefer_ascii nkf_compat |= O_PREFER_ASC
 /* -------------------------------------------------------------- */
@@ -1801,11 +1813,11 @@ unsigned long skf_get_valid_langcode P_((unsigned long,unsigned long));
 #define codeset_sjis78	24 /* JIS X 0208(1978) SJIS version	  */
 #define codeset_sjiscl	25 /* SJIS for cellular phone		  */
 #define	codeset_cp932w	26  /* MS cp932 (WideByteCharConv compat) */
-#define	codeset_cp20932	27  /* MS cp20932 (hairy euc-jp)	  */
-#define	codeset_cp51932	28  /* MS cp51932 (cp932'd euc-jp)	  */
-#define	codeset_cp5022x	29  /* MS cp50220 (cp932'd jis x208)	  */
-#define	codeset_cp50221	30  /* MS cp50221 (cp932'd jis x208)	  */
-#define	codeset_cp50222	31  /* MS cp50222 (cp932'd jis x208)	  */
+#define	codeset_cp51932	27  /* MS cp51932 (cp932'd euc-jp)	  */
+#define	codeset_cp5022x	28  /* MS cp50220 (cp932'd jis x208)	  */
+#define	codeset_cp50221	29  /* MS cp50221 (cp932'd jis x208)	  */
+#define	codeset_cp50222	30  /* MS cp50222 (cp932'd jis x208)	  */
+#define	codeset_cp20932	31  /* MS cp20932 (hairy euc-jp)	  */
 #define codeset_utf16le	32 /* UTF-16LE (dummy entry)		  */
 #define codeset_utf16be	33 /* UTF-16BE (dummy entry)		  */
 #define codeset_utf8	34 /* UTF8 (dummy entry)		  */
@@ -1898,13 +1910,14 @@ unsigned long skf_get_valid_langcode P_((unsigned long,unsigned long));
 #define codeset_sjisontt 121 /* Shift JIS with old-ntt glyph	  */
 #define	codeset_nyukan_utf8 122  /* nyukan gaiji utf8		  */
 #define	codeset_nyukan_utf16 123  /* nyukan gaiji utf8		  */
-#define	codeset_binary	 124  /* no codeset is given		  */
-#define	codeset_locale	 125  /* no codeset is given		  */
+#define codeset_koi8u	 124  /* GL:ascii, GR:koi-8-Ukrainean	  */
+#define	codeset_binary	 125  /* no codeset is given		  */
+#define	codeset_locale	 126  /* no codeset is given		  */
 #ifdef ARIB_SUPPORT
-#define	codeset_aribb24	 126  /* ARIB-B24-SI			  */
-#define codeset_end	 127 /* psuedo code: end-point		  */
+#define	codeset_aribb24	 127  /* ARIB-B24-SI			  */
+#define codeset_end	 128 /* psuedo code: end-point		  */
 #else
-#define codeset_end	 126 /* psuedo code: end-point		  */
+#define codeset_end	 127 /* psuedo code: end-point		  */
 #endif
 
 /* -------------------------------------------------------------- */
@@ -1955,6 +1968,7 @@ extern const struct long_option codeset_option_code[];
 /* -------------------------------------------------------------- */
 extern int	cname_comp	P_ ((char *, char *));
 extern int 	skf_search_cname P_((char *));
+extern int 	skf_search_cname_w_alias P_((char *));
 extern int 	skf_search_chname P_((char *));
 extern int	skf_option_parser \
 			P_((char *,const struct long_option *));
@@ -2116,6 +2130,9 @@ extern void	utf8_finish_procedure();
 extern void	ucod_finish_procedure();
 extern void	BG_finish_procedure();
 extern void	BRGT_finish_procedure();
+extern void	KEIS_finish_procedure();
+extern void	puny_finish_procedure();
+extern void	GBKR_finish_procedure();
 
 /* -------------------------------------------------------------- */
 /* windows filename limitation hooker				  */
