@@ -23,7 +23,7 @@
 **********************************************************************
 	skf_fileio.c:	various file io routines
 		v1.x	for skf v1.x
-	$Id: skf_fileio.c,v 1.49 2015/04/26 15:08:38 seiji Exp seiji $
+	$Id: skf_fileio.c,v 1.51 2016/06/11 13:43:55 seiji Exp seiji $
 */
 
 #ifndef _POSIX_SOURCE
@@ -78,7 +78,7 @@
 #include "oconv.h"
 
 #ifdef NEED_BINMODE
-#if defined(WIN32) || defined(SKF_MINGW)
+#if defined(_WIN32) || defined(SKF_MINGW)
 #include <io.h>
 #endif
 const char *skf_inmode = "rb";
@@ -95,6 +95,9 @@ static	short   hold_pntr = 0;
 
 static  int	hold_buf[DEFAULT_HOLD_SIZE];
 
+#ifdef USE_FWRITE
+int	fodisc;
+#endif
 /* --------------------------------------------------------------- */
 /* hold stack control: This is treated as Queue			   */
 /* Memo: Since program pushes at most 2-byte without test, at	   */
@@ -129,7 +132,7 @@ void Qflush()
 /* --------------------------------------------------------------- */
 /* buffers							   */
 /* --------------------------------------------------------------- */
-unsigned char *stdibuf;
+unsigned char *stdibuf = NULL;
 
 /* --------------------------------------------------------------- */
 /* SWIG								   */
@@ -159,7 +162,7 @@ int	*p;		/* file descripter			  */
 /* buffers							   */
 /* --------------------------------------------------------------- */
 unsigned char stdobuf[O_BUFSIZ];
-unsigned char *stdibuf;
+unsigned char *stdibuf = NULL;
 
 #if defined(FAST_GETC)
 
@@ -196,8 +199,7 @@ int	*p;		/* file descripter			  */
 	/* Maybe I should pause for signals, but this one is for  */
 	/* portability by sacrificing performance.		  */
 
-	buf_p = (int) read(*p,stdibuf,(size_t)I_BUFSIZ);
-	while (buf_p < 0) {
+	while ((buf_p = (int) read(*p,stdibuf,(size_t)I_BUFSIZ)) < 0) {
 	    skf_readerr(errno);
 	    if (errno != EAGAIN) {
 		return(sABRT);
@@ -213,11 +215,15 @@ int	*p;		/* file descripter			  */
 void skf_ioinit(fout)
 skfoFILE *fout;	/* stdout */
 {
-    if ((stdibuf = calloc((size_t)I_BUFSIZ,sizeof(unsigned char)))
-	    == NULL) {
-	skferr(SKF_MALLOCERR,(long)1,(long)0);
-	skf_exit(EXIT_FAILURE);
-    };
+    if (stdibuf == NULL) {
+	if ((stdibuf = calloc((size_t)I_BUFSIZ,sizeof(unsigned char)))
+		== NULL) {
+	    skferr(SKF_MALLOCERR,(long)1,(long)0);
+	    skf_exit(EXIT_FAILURE);
+	};
+    } else;
+
+    Qflush();
 #ifndef USE_FWRITE
     skf_setvbuf((FILE *)fout, (char *)stdobuf, O_BUFSIZ);
 #endif
@@ -246,6 +252,18 @@ skfoFILE	*f;
     obuf_p = 0;
 }
 
+skfoFILE *skf_fdopen(fp,typ)
+int fp;
+const char *typ;
+{
+#ifdef USE_FWRITE
+    fodisc = fp;
+    return(&fodisc);
+#else
+    return(fdopen(fp,typ));
+#endif
+}
+
 /* output file_output			*/
 int skf_oflush(ch)
 int ch;
@@ -264,11 +282,14 @@ char *mode;
 {
 	FILE *ifi;
 
-	if ((stdibuf = calloc((size_t)I_BUFSIZ,sizeof(unsigned char)))
-		== NULL) {
-	    skferr(SKF_MALLOCERR,(long)1,(long)0);
-	    skf_exit(EXIT_FAILURE);
-	};
+	if (stdibuf == NULL) {
+	    if ((stdibuf = calloc((size_t)I_BUFSIZ,sizeof(unsigned char)))
+		    == NULL) {
+		skferr(SKF_MALLOCERR,(long)1,(long)0);
+		skf_exit(EXIT_FAILURE);
+	    };
+	} else;
+
 	ifi = fopen(name,mode);
 	return(ifi);
 }
@@ -285,6 +306,10 @@ skfFILE *f;
 void skf_ioinit(fout)
 skfoFILE *fout;	/* stdout */
 {
+
+/* --- initialize buffer pointer --------------------------------- */
+    Qflush();
+
 #ifndef	USE_FWRITE
     skf_setvbuf((FILE *)fout, (char *)stdobuf, O_BUFSIZ);
 #endif
@@ -417,7 +442,7 @@ FILE *fp;
 int mode;
 char *str;
 {
-#if defined(WIN32) || defined(SKF_MINGW)
+#if defined(_WIN32) || defined(SKF_MINGW)
     if (setmode(fileno(fp),O_BINARY) < 0) {
     	skf_openerr(str,1);
 	skf_exit(EXIT_FAILURE);
