@@ -35,6 +35,7 @@ extern int    hold_size;
 /* Memo: Since program pushes at most 2-byte without check, 	  */
 /*	 2 space is reserved when checking queue is full.	  */
 #define		Qempty  (hold_size <= 0)
+#define		Qle2    (hold_size <= 2)
 #define		Qfull   ((hold_size >= DEFAULT_HOLD_SIZE - 2) \
 			|| ((unbuf_f) && (hold_size > 0)))
 
@@ -201,6 +202,7 @@ typedef FILE	skfoFILE; /* output side file type		  */
 
 extern 		skfoFILE *fout;
 extern		void	SKFfflush P_((skfoFILE *));
+extern	skfoFILE  *skf_fdopen P_((int,const char *));
 extern	void	skf_ioinit P_((skfoFILE *));
 
 #else			/* not fast_getc mode			  */
@@ -222,6 +224,7 @@ extern int	rGETC P_((skfFILE *));
 #define		unGETC(c,p)	ungetc(c,p)
 extern FILE	*skf_fopen P_((char *,char *));
 #define		skf_fclose(fp)	fclose(fp)
+#define		skf_fdopen(fp,typ) fdopen(fp,typ)
 
 /* -------------------------------------------------------------- */
 /* putchar substitution 					  */
