@@ -23,7 +23,7 @@
 **********************************************************************
     ucodoconv.c: output converters for Unicode(TM) variants
 	v1.90	Newly written from unic*de.c parsing
-    $Id: ucodoconv.c,v 1.80 2015/04/26 15:08:38 seiji Exp seiji $
+    $Id: ucodoconv.c,v 1.83 2016/06/14 14:55:50 seiji Exp seiji $
 **/
 
 #include <stdio.h>
@@ -48,13 +48,10 @@ static int	utf7_res = 0;	/* utf-7 amari			   */
 static void SKFUTF7ENCODE P_((skf_ucode));
 static void encode_pslenset P_((int *,int *,skf_ucode));
 
-static char ucod_cbuf[32];	/* uri conversion buffer	   */
-
 #ifdef ACE_SUPPORT
-static int  is_prohibit P_((skf_ucode));
+int  is_prohibit P_((skf_ucode));
 #endif
 
-extern char *utf8_urioutstr	P_((skf_ucode));
 /* --- macros ---------------------------------------------------- */
 #define utf8put4(x) SKFputc((int)(ZF4VAL + ((x & ZL4MSK) >> 18))); \
     SKFputc((int)(ZF2VAL + ((x & ZL43MSK) >> 12)));\
@@ -116,8 +113,10 @@ skf_ucode	c3;
 #ifdef ACE_SUPPORT
     } else if (is_ucs_puny(conv_cap)) {
     	if (!is_prohibit(c3) || is_puny_strdelim(c3)) o_p_encode(c3);
-	else out_undefined(c3,SKF_OUT_PROHIBIT);
-		/* idn prohibit table (rfc3454) is discarded	   */
+	else { /* idn prohibit table (rfc3454) is discarded	   */
+	    o_p_encode(A_SP);
+	    out_undefined(c3,SKF_OUT_PROHIBIT);
+	};
 #endif
     } else if (	/* UTF7: this routine loops due to 0x26	*/
 	   ((c3 >= 0x21) && (c3 <= 0x26))
@@ -148,7 +147,7 @@ void SKFUNISTROUT(st)
 const char *st;
 {
     int len;
-    for (len=0; (len<24) && (*st != '\0') ; len++,st++) {
+    for (len=0; (len<30) && (*st != '\0') ; len++,st++) {
 	SKFUNI1OUT((skf_ucode)(*st));
     };
 }
@@ -217,8 +216,10 @@ void UNI_latin_oconv (ch)
 #ifdef ACE_SUPPORT
     } else if (is_ucs_puny(conv_cap)) {
     	if (!is_prohibit(ch)) o_p_encode(ch);
-	else out_undefined(ch,SKF_OUT_PROHIBIT);
-		/* idn prohibit table (rfch454) is discarded	   */
+	else { /* idn prohibit table (rfch454) is discarded	   */
+	    o_p_encode(A_SP);
+	    out_undefined(ch,SKF_OUT_PROHIBIT);
+	};
 #endif
     } else if (is_ucs_utf7(conv_cap)) {
 	if (!is_utf7_shift) {
@@ -282,8 +283,10 @@ void UNI_cjkkana_oconv (ch)
 #ifdef ACE_SUPPORT
     } else if (is_ucs_puny(conv_cap)) {
     	if (!is_prohibit(ch)) o_p_encode(ch);
-	else out_undefined(ch,SKF_OUT_PROHIBIT);
-		/* idn prohibit table (rfc3454) is discarded	   */
+	else { /* idn prohibit table (rfc3454) is discarded	   */
+	    o_p_encode(A_SP);
+	    out_undefined(ch,SKF_OUT_PROHIBIT);
+	};
 #endif
     } else;
 }
@@ -342,8 +345,10 @@ skf_ucode c1;
 #ifdef ACE_SUPPORT
     } else if (is_ucs_puny(conv_cap)) {
     	if (!is_prohibit(c1)) o_p_encode(c1);
-	else out_undefined(c1,SKF_OUT_PROHIBIT);
-		/* idn prohibit table (rfc3454) is discarded	   */
+	else { /* idn prohibit table (rfc3454) is discarded	   */
+	    o_p_encode(A_SP);
+	    out_undefined(c1,SKF_OUT_PROHIBIT);
+	};
 #endif
     };
 }
@@ -428,8 +433,10 @@ skf_ucode ch;
 #ifdef ACE_SUPPORT
     } else if (is_ucs_puny(conv_cap)) {
     	if (!is_prohibit(ch)) o_p_encode(ch);
-	else out_undefined(ch,SKF_OUT_PROHIBIT);
-		/* idn prohibit table (rfc3454) is discarded	   */
+	else { /* idn prohibit table (rfc3454) is discarded	   */
+	    o_p_encode(A_SP);
+	    out_undefined(ch,SKF_OUT_PROHIBIT);
+	};
 #endif
     } else {
 	if ((ch <= UNI_HIGH_LIMIT) && (ch >= 0x10000) && !limit_ucs2) {
@@ -490,8 +497,10 @@ skf_ucode c1;
 #ifdef ACE_SUPPORT
     } else if (is_ucs_puny(conv_cap)) {
     	if (!is_prohibit(c1) || force_private_idn_o) o_p_encode(c1);
-	else out_undefined(c1,SKF_OUT_PROHIBIT);
-		/* idn prohibit table (rfc3454) is discarded	   */
+	else { /* idn prohibit table (rfc3454) is discarded	   */
+	    o_p_encode(A_SP);
+	    out_undefined(c1,SKF_OUT_PROHIBIT);
+	};
 #endif
     } else {
 	utf8put3(c1);
@@ -539,6 +548,9 @@ void UNI_compat_oconv (c1)
     		/* nkf doesn't use fullwidth Yen sign		   */
     	post_oconv(0xa5);
 	return;
+    } else if (c1 >= 0xfffe) {
+	out_undefined(c1,SKF_OUNDEF);
+	return;
     } else if ((use_compat) && 
 	((!sup_cjk_cmp) || (c1 < 0xfa30) || (c1 > 0xfa6a))) {
 	    c4 = c2;
@@ -616,8 +628,10 @@ void UNI_compat_oconv (c1)
 #ifdef ACE_SUPPORT
     } else if (is_ucs_puny(conv_cap)) {
     	if (!is_prohibit(c1)) o_p_encode(c1);
-	else out_undefined(c1,SKF_OUT_PROHIBIT);
-		/* idn prohibit table (rfc3454) is discarded	   */
+	else { /* idn prohibit table (rfc3454) is discarded	   */
+	    o_p_encode(A_SP);
+	    out_undefined(c1,SKF_OUT_PROHIBIT);
+	};
 #endif
     };
 }
@@ -686,44 +700,32 @@ void utf8_finish_procedure()
 }
 
 /* --- output utilities ------------------------------------------ */
-/*@-globstate@*//*@-statictrans@*/
-char *utf8_urioutstr(x)
+void utf8_urioutstr(x)
 skf_ucode x;
 {
-    if (x < 0x80) {
-    	(void)snprintf(ucod_cbuf,1,"%c",x);
-    } else if (x < 0x800) {
-	(void)snprintf(ucod_cbuf,6,"\%02x\%02x",
-		(ZF23VAL + ((x & ZL23MSK) >> 6)),
-		((x & Z1MSK) | ZCVAL));
-    } else if (x < 0x10000) {
-	(void)snprintf(ucod_cbuf,9,"\%02x\%02x\%02x",
-		(ZF3VAL + ((x & ZL3MSK) >> 12)),
-		(ZF2VAL + ((x & ZL2MSK) >> 6)),
-		((x & Z1MSK) | ZCVAL));
-    } else if ((x <= 0x10ffff) && (x >= 0x10000) && !limit_ucs2) {
-	(void)snprintf(ucod_cbuf,12,"\%02lx\%02lx\%02x\%02x",
-		(ZF4VAL + ((x & ZL4MSK) >> 18)),
-		(ZF2VAL + ((x & ZL43MSK) >> 12)),
-		(ZF2VAL + ((x & ZL2MSK) >> 6)),
-		((x & Z1MSK) | ZCVAL));
-    } else ;
-
-    return(ucod_cbuf);
+    SKFputc(0x25);
+    SKFputc(hex_conv_table[(x >> 4) & 0x0fU]);
+    SKFputc(hex_conv_table[(x & 0x0fU)]);
 }
 
 void utf8_uriout(x)
 skf_ucode x;
 {
-    char *sym;
-
-    if ((sym = utf8_urioutstr(x)) != NULL) {
-    	while (*sym == '\0') {
-	    SKFputc(*sym++);
-	};
-    } else {
-	out_undefined(x,SKF_OUNDEF);
-    };
+    if (x < 0x80) {
+    	utf8_urioutstr(x);
+    } else if (x < 0x800) {
+    	utf8_urioutstr(ZF23VAL + ((x & ZL23MSK) >> 6));
+    	utf8_urioutstr((x & Z1MSK) | ZCVAL);
+    } else if (x < 0x10000) {
+    	utf8_urioutstr(ZF3VAL + ((x & ZL3MSK) >> 12));
+    	utf8_urioutstr(ZF2VAL + ((x & ZL2MSK) >> 6));
+    	utf8_urioutstr((x & Z1MSK) | ZCVAL);
+    } else if ((x <= 0x10ffff) && (x >= 0x10000) && !limit_ucs2) {
+    	utf8_urioutstr(ZF4VAL + ((x & ZL4MSK) >> 18));
+    	utf8_urioutstr(ZF2VAL + ((x & ZL43MSK) >> 12));
+    	utf8_urioutstr(ZF2VAL + ((x & ZL2MSK) >> 6));
+    	utf8_urioutstr(((x & Z1MSK) | ZCVAL));
+    } else ;
 }
 
 #ifdef ACE_SUPPORT
@@ -740,6 +742,14 @@ skf_ucode ch;
     else if (((ch >= 0x2000) && (ch <= 0x200f))
 	|| ((ch >= 0x2028) && (ch <= 0x202f))
     	|| (ch == 0x205f) || (ch == 0x3000)) return(1);
+    /* mapped to nothing */
+    else if ((ch == 0xad) || (ch == 0x1806) || (ch == 0x2006)
+	|| (ch == 0x034f) || (ch == 0xfeff) 
+	|| ((ch >= 0x180b) && (ch <= 0x180d))
+	|| ((ch >= 0x200b) && (ch <= 0x200d))
+	|| ((ch >= 0xfe00) && (ch <= 0xfe0f))
+	|| ((ch >= 0x1d173) && (ch <= 0x1d17a))
+	) return(2);
     /* controls */
     else if ((ch == A_DEL) || (ch == 0x06dd) || (ch == 0x070f)
 	|| (ch == 0x180e) || (ch == 0xfeff) 
@@ -857,7 +867,7 @@ skf_ucode cc;		/* table lookup result */
 					/* variation selector	   */
 		    if (enable_var_cntl) 
 			encode_pslenset(&plen,&slen,cc);
-		} else if ((use_compat) && 
+		} else if ((use_compat) && (c3 == 0xfa) &&
 		    ((!sup_cjk_cmp) || (cc < 0xfa30) || (cc > 0xfa6a))) {
 		    encode_pslenset(&plen,&slen,cc);
 		} else if ((c3 >= 0xf9) && (c3 < 0xfb)) {
